using System;
using System.Collections.ObjectModel;
using System.Linq;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// A BlockGenerator for elements from the Dataflow model.
    /// </summary>
    public abstract class DataFlowBlockGenerationBase : BlockGeneratorBase
    {
        protected FlowElement flowElement;
        protected UniqueNameProvider nameProvider;

        /// <summary>
        /// Initializes a new instance of the <see cref="DataFlowBlockGenerationBase"/> class.
        /// </summary>
        /// <param name="flowElement">The flow element.</param>
        /// <param name="nameProvider">The name provider.</param>
        /// <param name="hardware">The HardwareDescriptionModel.</param>
        protected DataFlowBlockGenerationBase(FlowElement flowElement, UniqueNameProvider nameProvider, HardwareDescriptionModel hardware) : base(hardware)
        {
            this.flowElement = flowElement;
            this.nameProvider = nameProvider;
        }

        #region links
        private ReadOnlyCollection<FlowElementReferencesTargetFlowElements> incommingLinks;
        /// <summary>
        /// Gets the incomming link to the field with the given guid.
        /// </summary>
        protected FlowElementReferencesTargetFlowElements IncomingLink(Guid field)
        {
            return IncomingLinks().FirstOrDefault(x => x.TargetEntry.Equals(field));
        }
        /// <summary>
        /// Gets all incomming links to the FlowElement of this generator.
        /// </summary>
        protected ReadOnlyCollection<FlowElementReferencesTargetFlowElements> IncomingLinks()
        {
            if (incommingLinks == null)
                incommingLinks = FlowElementReferencesTargetFlowElements.GetLinksToSourceFlowElements(flowElement);

            return incommingLinks;
        }

        private ReadOnlyCollection<FlowElementReferencesTargetFlowElements> outgoingLinks;
        /// <summary>
        /// Gets the outgoing link from the field with the given guid.
        /// </summary>
        protected FlowElementReferencesTargetFlowElements OutgoingLink(Guid field)
        {
            return OutgoingLinks().FirstOrDefault(x => x.SourceEntry.Equals(field));
        }
        /// <summary>
        /// Gets all outgoing links from the FlowElement of this generator.
        /// </summary>
        protected ReadOnlyCollection<FlowElementReferencesTargetFlowElements> OutgoingLinks()
        {
            if (outgoingLinks == null)
                outgoingLinks = FlowElementReferencesTargetFlowElements.GetLinksToTargetFlowElements(flowElement);

            return outgoingLinks;
        }
        #endregion
    }
}
