using System;
using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Ways of adding a Argument to the list of arguemnts used for a functiuon call
    /// </summary>
    public enum AddCallArgumentType
    {
        /// <summary>
        /// a literal
        /// </summary>
        Value,    
        /// <summary>
        /// a variable name
        /// </summary>
        ByValue,  
        /// <summary>
        /// a pointer of a variablen
        /// </summary>
        ByReference
    }

    /// <summary>
    /// Represents the arguments that will be used for a function call.
    /// </summary>
    public class CallArguments
    {
        private int maxPos = 1;
        private Dictionary<int, string> args = new Dictionary<int, string>();

        /// <summary>
        /// Adds a new argument to the list at a given position
        /// </summary>
        /// <param name="pos">The pos the argument will be added.</param>
        /// <param name="arg">The value of the argument.</param>
        /// <param name="addingType">Type of the argument.</param>
        public void Add(int pos, string arg, AddCallArgumentType addingType)
        {
            string s;
            switch (addingType)
            {
                case AddCallArgumentType.Value:
                    s = arg;  
                    break;
                case AddCallArgumentType.ByValue:
                    s = arg;
                    break;
                case AddCallArgumentType.ByReference:
                    s = "&" + arg;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("addingType");
            }

            // remember the highest value of pos
            maxPos = Math.Max(maxPos, pos);

            args.Add(pos,s);
        }

        /// <summary>
        /// Adds a new argument to the end of the list
        /// </summary>
        /// <param name="arg">The arg.</param>
        /// <param name="arg">The value of the argument.</param>
        /// <param name="addingType">Type of the argument.</param>
        public void Add(string arg, AddCallArgumentType addingType)
        {
            Add(maxPos + 1, arg, addingType);
        }

        /// <summary>
        /// Returns the arguemnt list in a way that can be 
        /// inserted int the function call.
        /// </summary>
        /// <returns></returns>
        public string GetArgumentList()
        {
            var parameter = from p in args orderby p.Key select p.Value;
            return string.Join(", ", parameter.ToArray());
        }
    }
}
