using System;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.HardwareDescription;
using Variable=BenjaminSchroeter.Flow.DataFlow.Variable;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Generates code for a Variable defined in the Hardware Description Model
    /// </summary>
    internal class HardwareVariableGenerator : DataFlowBlockGenerationBase
    {
        private new Variable flowElement;
        private BenjaminSchroeter.Flow.HardwareDescription.Variable hardwareDefinition;

        /// <summary>
        /// Initializes a new instance of the <see cref="HardwareVariableGenerator"/> class.
        /// </summary>
        public HardwareVariableGenerator(Variable flowElement, UniqueNameProvider nameProvider, HardwareDescriptionModel hardware) : base(flowElement, nameProvider, hardware)
        {
            this.flowElement = flowElement;
            this.hardwareDefinition = flowElement.Definition(hardware) as BenjaminSchroeter.Flow.HardwareDescription.Variable;
        }

        public override void generateBehavior()
        {
            // each of this functions only generates something
            // if there is a incomming / outgoing link
            GenerateSet();
            GenerateGet();
        }

        /// <summary>
        /// Generates the set of a value to the variable if there is an incomming link.
        /// </summary>
        private void GenerateSet()
        {
            // find incommong link
            FlowElementReferencesTargetFlowElements link = IncomingLink(Guid.Empty);
            if (link == null)
                return;

            string variable = nameProvider.GetSourceName(link);

            FunctionCall(hardwareDefinition.CallParameterSetter, variable, flowElement.Type.AsDslType(flowElement.Store));
        }

        /// <summary>
        /// Generates the get of a value to the variable if there is an outgoing link.
        /// </summary>
        private void GenerateGet()
        {
            // any outgoing links?
            if (flowElement.TargetFlowElements.Count == 0)
                return;

            string variable = nameProvider.GetName(flowElement);

            CallParameter call = hardwareDefinition.CallParameterGetter;
            if (call.HasFunction)
            {
                VariableDeclaration(flowElement.Type, flowElement.Store, variable);
                FunctionCall(call, variable, flowElement.Type.AsDslType(flowElement.Store));
            }
        }

    }
}