using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.DataTypes;
using BenjaminSchroeter.Flow.HardwareDescription;
using Input=BenjaminSchroeter.Flow.DataFlow.Input;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Generates the code for inputs
    /// </summary>
    public class InputGenerator : DataFlowBlockGenerationBase
    {
        private new Input flowElement;
        private BenjaminSchroeter.Flow.HardwareDescription.Input hardwareDefinition;

        /// <summary>
        /// Initializes a new instance of the <see cref="InputGenerator"/> class.
        /// </summary>
        public InputGenerator(Input flowElement, UniqueNameProvider nameProvider, HardwareDescriptionModel hardware) : base(flowElement, nameProvider, hardware)
        {
            this.flowElement = flowElement;
            this.hardwareDefinition = flowElement.Definition(hardware) as BenjaminSchroeter.Flow.HardwareDescription.Input;
        }

        /// <summary>
        /// Generates the behavior.
        /// </summary>
        public override void generateBehavior()
        {
            CallParameter mainCall = hardwareDefinition.CallParameter;
            CallArguments mainCallArgs = new CallArguments();

            bool generateMailCall = false;
            Dictionary<string, DataTypeString> variablesForMainCall = new Dictionary<string, DataTypeString>();
            foreach (InputField field in hardwareDefinition.Fields)
            {
                Field dataFlowField = flowElement.Fields.FirstOrDefault(f => f.Guid.Equals(field.Guid));
                string fieldVariable;
                if (dataFlowField != null)
                    fieldVariable = nameProvider.GetName(dataFlowField);
                else
                    fieldVariable = nameProvider.GetName(field.Guid, "dummy"); // this variable is not needed anywhere, but for the function call

                CallParameter fieldCall = field.CallParameter;
                if (!fieldCall.HasFunction && fieldCall.HasParameter)
                {
                    variablesForMainCall.Add(fieldVariable, field.Type);
                    mainCallArgs.Add(fieldCall.ParameterPos, fieldVariable, AddCallArgumentType.ByReference);
                    if (dataFlowField != null && OutgoingLink(dataFlowField.Guid) != null)
                        generateMailCall = true;
                }

                if (fieldCall.HasFunction && dataFlowField != null && OutgoingLink(dataFlowField.Guid) != null)
                {
                    VariableDeclaration(field.Type, field.Store, fieldVariable); 
                    FunctionCall(fieldCall, fieldVariable, field.Type.AsDslType(field.Store));
                }
            }

            if ( generateMailCall )
            {
                foreach (KeyValuePair<string, DataTypeString> pair in variablesForMainCall)
                    VariableDeclaration(pair.Value, flowElement.Store, pair.Key);
                FunctionCall(mainCall,mainCallArgs);
            }
        }
    }
}