using System;
using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Generates the code for the boolean operations used in the DataFlow
    /// </summary>
    public class OperationGenerator : DataFlowBlockGenerationBase
    {
        private new Operation flowElement;

        /// <summary>
        /// Initializes a new instance of the <see cref="OperationGenerator"/> class.
        /// </summary>
        public OperationGenerator(Operation flowElement, UniqueNameProvider nameProvider, HardwareDescriptionModel hardware)
            : base(flowElement, nameProvider, hardware)
        {
            this.flowElement = flowElement;
        }

        /// <summary>
        /// Generates the behavior.
        /// </summary>
        public override void generateBehavior()
        {
            // declare a variable
            string variable = nameProvider.GetName(flowElement);

            string typename = DataTypeHelper.GetDataType(flowElement,null);
            DataType d = this.hardware.DataTypes.First(x => x.Name == typename);
            sb.AppendFormatLine("{0} {1};", d.CType, variable);  // always a boolean type

            // get the incomming links
            List<string> inputs = new List<string>();
            foreach (FlowElementReferencesTargetFlowElements link in IncomingLinks())
            {
                // the head of an Input Event represents always a "true" value
                if ( link.SourceFlowElement is InputEvent && link.SourceEntry.Equals(Guid.Empty))
                    inputs.Add("true");
                else
                    inputs.Add(nameProvider.GetSourceName(link));
            }


            string op = null;
            sb.AppendFormat("{0} = ", variable);

            // get the c operator of the operation
            switch (flowElement.Kind)
            {
                case OperationKind.LogicalNot:
                    sb.AppendFormat("!{0}", inputs[0]);
                    break;
                case OperationKind.LogicalAnd:
                    op = " && ";
                    break;
                case OperationKind.LogicalOr:
                    op = " || ";
                    break;
                case OperationKind.LogicalXor:
                    op = " ^ ";
                    break;
                case OperationKind.True:
                    sb.Append("true");
                    break;
                case OperationKind.False:
                    sb.Append("false");
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            // join all inputs by this operator
            if (op != null)
                sb.Append(string.Join(op, inputs.ToArray()));

            sb.AppendLine(";");
        }
    }
}
