using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.HardwareDescription;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using Output=BenjaminSchroeter.Flow.DataFlow.Output;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Generates the code for outputs.
    /// </summary>
    public class OutputGenerator : DataFlowBlockGenerationBase
    {
        private new Output flowElement;
        private BenjaminSchroeter.Flow.HardwareDescription.Output hardwareDefinition;

        /// <summary>
        /// Initializes a new instance of the <see cref="OutputGenerator"/> class.
        /// </summary>
        public OutputGenerator(Output flowElement, UniqueNameProvider nameProvider, HardwareDescriptionModel hardware) : base(flowElement, nameProvider, hardware)
        {
            this.flowElement = flowElement;
            this.hardwareDefinition = flowElement.Definition(hardware) as BenjaminSchroeter.Flow.HardwareDescription.Output;
        }

        /// <summary>
        /// Generates the behavior.
        /// </summary>
        public override void generateBehavior()
        {
            CallParameter mainCall = hardwareDefinition.CallParameter;
            CallArguments mainCallArgs = new CallArguments();

            foreach (OutputField field in hardwareDefinition.Fields)
            {
                // find incommong link for this field
                FlowElementReferencesTargetFlowElements link = IncomingLink(field.Guid);
                if (link == null)
                    continue;

                string fieldVariable = nameProvider.GetSourceName(link);

                CallParameter fieldCall = field.CallParameter;
                
                AddCallArgumentType argumentType = AddCallArgumentType.ByValue;
                if (fieldCall.ParameterReferenceType != "" && field.Type.AsDslType(field.Store) != DslDataType.String)
                    argumentType = AddCallArgumentType.ByReference;

                if (!fieldCall.HasFunction && fieldCall.HasParameter)
                    mainCallArgs.Add(fieldCall.ParameterPos, fieldVariable, argumentType);

                FunctionCall(fieldCall, fieldVariable, field.Type.AsDslType(field.Store));
            }

            FunctionCall(mainCall, mainCallArgs);
        }
    }
}