using System.Collections.Generic;
using System.Linq;
using System.Text;
using BenjaminSchroeter.Flow.HardwareDescription;
using BenjaminSchroeter.Flow.HardwareDescription.Data;

namespace BenjaminSchroeter.Flow.CodeGen
{
    /// <summary>
    /// Generates the init calls for elements of the hardware description
    /// </summary>
    public class HardwareInitGenerator : BlockGeneratorBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HardwareInitGenerator"/> class.
        /// </summary>
        public HardwareInitGenerator(HardwareDescriptionModel hardware) : base(hardware)
        {
        }

        /// <summary>
        /// Generates the init calls for the given Port.
        /// </summary>
        public string GenerateInit(Port def)
        {
            // only Inputs and Outputs need an InitCall
            if (!(def is Input) && !(def is Output))
                return "";

            sb = new StringBuilder();

            CallParameter call = def.CallParameterInit;

            // if no init call is defined
            if (!call.HasFunction)
                return "";
            
            Comment("Init for " + def.ToString());

            // find all parameters
            CallArguments args = new CallArguments();
            if (call.HasEnabledParameter)
                args.Add(call.EnabledParameterPos, ParameterValue(def.Enabled), AddCallArgumentType.Value); 

            foreach (Parameter parameter in def.Parameters)
            {
                CallParameter c = parameter.CallParameter;
                if (c.HasParameter)
                {
                    DslDataType dataType = parameter.Type.AsDslType(parameter.Store);
                    args.Add(c.ParameterPos, ParameterValue(parameter.Value, dataType) , AddCallArgumentType.Value);
                }
            }

            // find the enabled parameter flags
            IEnumerable<EnableableFieldBase> fields;
            if (def is Input)
                fields = from f in (def as Input).Fields select f as EnableableFieldBase;
            else
                fields = from f in (def as Output).Fields select f as EnableableFieldBase;

            foreach (EnableableFieldBase field in fields)
            {
                CallParameter c = field.CallParameter;
                if (c.HasEnabledParameter)
                    args.Add(c.EnabledParameterPos, ParameterValue(field.Enabled), AddCallArgumentType.Value);
            }

            // generate the function call
            FunctionCall(call, args);

            return sb.ToString();
        }
    }
}
