using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Dsl.Graphs;

namespace BenjaminSchroeter.Flow.DataFlow.Graphs
{
    /// <summary>
    /// Representation of the elements of a Flow in 
    /// the DataFlow Model as a graph
    /// </summary>
    public class DataFlowGraph : Graph<FlowElement,FlowElementReferencesTargetFlowElements>
    {
        private IList<FlowElement> vertices;

        /// <summary>
        /// Initializes a new instance of the <see cref="DataFlowGraph"/> class.
        /// </summary>
        /// <param name="flow">The flow.</param>
        public DataFlowGraph(FlowBase flow)
        {
            this.vertices = flow.FlowElements.ToList();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DataFlowGraph"/> class.
        /// </summary>
        /// <param name="vertices">The vertices.</param>
        public DataFlowGraph(IList<FlowElement> vertices)
        {
            this.vertices = vertices;
        }

        /// <summary>
        /// All Verticeses of this Graph.
        /// </summary>
        /// <remarks>Thats all FlowElements of the Model / of the Flow</remarks>
        /// <returns></returns>
        public override IEnumerable<FlowElement> Vertices()
        {
            return vertices;
        }

        /// <summary>
        /// Children of the specified FlowElement.
        /// </summary>
        /// <param name="v">The v.</param>
        /// <returns></returns>
        public override IEnumerable<FlowElement> Children(FlowElement v)
        {
            return v.TargetFlowElements.Where(a => this.Vertices().Contains(a));
        }

        /// <summary>
        /// Gets a value indicating whether this instance is directed.
        /// </summary>
        /// <value>always true</value>
        public override bool IsDirected
        {
            get { return true; }
        }

        /// <summary>
        /// Removes the given vertice.
        /// </summary>
        /// <remarks>
        /// This can be used after creation of the graph but before using it.
        /// </remarks>
        /// <param name="toRemove">To remove.</param>
        public void RemoveVertice(FlowElement toRemove)
        {
            if (toRemove == null)
                return;

            this.vertices.Remove(toRemove);
        }
    }
}