using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.DataFlow.Graphs
{
    /// <summary>
    /// Representation of the elements of a Flow in 
    /// the DataFlow Model as a undirected graph
    /// </summary>
    public class UndirectedDataFlowGraph : DataFlowGraph
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="UndirectedDataFlowGraph"/> class.
        /// </summary>
        /// <param name="flow">The flow.</param>
        public UndirectedDataFlowGraph(FlowBase flow) : base(flow)
        {
        }

        /// <summary>
        /// Childrens of the specified FlowElement.
        /// </summary>
        /// <param name="v">The v.</param>
        /// <returns></returns>
        public override IEnumerable<FlowElement> Children(FlowElement v)
        {
            return v.TargetFlowElements.Union(v.SourceFlowElements).Where(a => this.Vertices().Contains(a));
        }

        /// <summary>
        /// Gets a value indicating whether this instance is directed.
        /// </summary>
        /// <value>always false</value>
        public override bool IsDirected
        {
            get
            {
                return false;
            }
        }
    }
}
