using System;
using System.Collections.Generic;
using System.Linq.Dynamic;
using System.Linq.Expressions;
using BenjaminSchroeter.Flow.DataTypes;
using BenjaminSchroeter.Flow.HardwareDescription.Data;

namespace BenjaminSchroeter.Flow.DataFlow
{
    partial class Formula
    {
        private string type;
        internal bool typeUpToDate;

        /// <summary>
        /// Gets the DataType of this Formula shape.
        /// </summary>
        /// <remarks>
        /// If the current type is up to date the cached one will be returned. 
        /// Otherwise the formula will be parsed an a new type returned.
        /// </remarks>
        /// <returns></returns>
        public DataTypeString GetTypeValue()
        {
            if ( !typeUpToDate )
            {
                typeUpToDate = true;
                type = "";

                try
                {
                    this.ParseFormula();
                }
                catch // catch all errors form ParseFormula!
                {
                }                
            }
                
            return new DataTypeString(type);
        }

        /// <summary>
        /// Parses the formula to update the Datatype of this shape
        /// </summary>
        private void ParseFormula()
        {
            // create a (named) parameter list from the incomming connections
            List<ParameterExpression> parameters = new List<ParameterExpression>();
            foreach (FlowElementReferencesTargetFlowElements link in FlowElementReferencesTargetFlowElements.GetLinksToSourceFlowElements(this))
            {
                DslDataType dslDataType = link.Type.AsDslType(this.Store);
                Type t = DataTypeHelper.DslType2DotNet(dslDataType);
                parameters.Add(Expression.Parameter(t, link.TargetAnnotation));
            }

            // parse an receive the expression
            Expression ex = ParseExpression(parameters.ToArray(), null, this.FormulaText);

            // store the type of this expression in the local cache
            type = DataTypeHelper.GetTypeName(ex.Type, this);
            typeUpToDate = true;
        }

        /// <summary>
        /// Parses the formula with the given parameters and returns the expression tree.
        /// </summary>
        /// <param name="parameters">The parameters for the formula.</param>
        /// <param name="resultType">Type of the result should be casted to.</param>
        /// <param name="expression">The expression tree.</param>
        /// <param name="values">optional: The values of the parameters.</param>
        /// <returns></returns>
        private Expression ParseExpression(ParameterExpression[] parameters, Type resultType, string expression, params object[] values)
        {            
            ExpressionParser parser = new ExpressionParser(parameters, expression, values);
            Expression parse = parser.Parse(resultType);
            return parse;
        }

        /// <summary>
        /// Marks the current type as invalid and forces a reevaluate of the type the next time the type property is requestet.
        /// </summary>
        public void InvalidateType()
        {
            this.typeUpToDate = false;
        }
    }
    
}
