using System.Diagnostics;

namespace BenjaminSchroeter.Flow.DataFlow
{
    partial class Operation
    {
        /// <summary>
        /// Gets the text for this Operation based on the OperationKind.
        /// </summary>
        /// <returns></returns>
        public string GetTextValue()
        {
            switch (this.Kind)
            {
                case OperationKind.LogicalNot:
                    return "NOT";
                case OperationKind.LogicalAnd:
                    return "AND";
                case OperationKind.LogicalOr:
                    return "OR";
                case OperationKind.LogicalXor:
                    return "XOR";
                case OperationKind.True:
                    return "true";
                case OperationKind.False:
                    return "false";
                
                default:
                    Debug.Fail("Whats kind of operation is it?");
                    return "?";
            }
        }

        /// <summary>
        /// Test whether this Operation accepts incoming connections.
        /// </summary>
        /// <remarks>
        /// Every Kind has a maximum number of incomming calls.
        /// </remarks>
        /// <returns></returns>
        public bool AcceptIncomingConnection()
        {
            switch (this.Kind)
            {
                // only one input for NOT
                case OperationKind.LogicalNot:
                    return this.SourceFlowElements.Count < 1;
                
                // many (infinity) inputs for AND and OR
                case OperationKind.LogicalAnd:
                case OperationKind.LogicalOr:
                    return true;
                
                // two inputs for XOR
                case OperationKind.LogicalXor:
                    return this.SourceFlowElements.Count < 2;
                
                // no input for TRUE and FALSE
                case OperationKind.True:
                case OperationKind.False:
                    return false;
                
                default:
                    Debug.Fail("Whats kind of operation is it?");
                    return false;
            }
        }
    }
}

