using System.Collections.Generic;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Design;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Flow.DataFlow
{
    partial class DataFlowToolboxHelper
    {
        /// <summary>
        /// Returns a list of toolbox items for use with this DSL.
        /// 
        /// This method addes the items for the Operations to the ToolboxItems created by the base implementation
        /// </summary>
        /// <returns></returns>
        public override IList<ModelingToolboxItem> CreateToolboxItems()
        {
            IList<ModelingToolboxItem> toolboxItems = base.CreateToolboxItems();

            using(Store store = new Store())
            {
                store.LoadDomainModels(typeof(CoreDesignSurfaceDomainModel),typeof(DataFlowDomainModel));

                using (Transaction t = store.TransactionManager.BeginTransaction("creating prototypes"))
                {
                    const string tabName = "Operators";
                    toolboxItems.Add(CreateOperationToolboxItem(store, "logical NOT", tabName, OperationKind.LogicalNot));
                    toolboxItems.Add(CreateOperationToolboxItem(store, "logical AND", tabName, OperationKind.LogicalAnd));
                    toolboxItems.Add(CreateOperationToolboxItem(store, "logical OR", tabName, OperationKind.LogicalOr));
                    toolboxItems.Add(CreateOperationToolboxItem(store, "logical XOR", tabName, OperationKind.LogicalXor));

                    toolboxItems.Add(CreateOperationToolboxItem(store, "True", tabName, OperationKind.True));
                    toolboxItems.Add(CreateOperationToolboxItem(store, "False", tabName, OperationKind.False));

                    t.Rollback();
                }
            }

            return toolboxItems;
        }

        private ModelingToolboxItem CreateOperationToolboxItem(Store store, string itemName, string tabName, OperationKind operationKind)
        {
            Operation element = (Operation) store.ElementFactory.CreateElement(Operation.DomainClassId);
            element.Kind = operationKind;
            ElementGroup elementGroup = new ElementGroup(store.DefaultPartition);
            elementGroup.AddGraph(element, true);

            ModelingToolboxItem item = new ModelingToolboxItem(
                itemName, // Unique identifier (non-localized) for the toolbox item.
                1, // Position relative to other items in the same toolbox tab.
                itemName, // Localized display name for the item.
                Resource.ShapeToolBitmap , // Image displayed next to the toolbox item.
                tabName, // Unique identifier (non-localized) for the toolbox item tab.
                tabName, // Localized display name for the toolbox tab.
                "", // F1 help keyword for the toolbox item.
                "", // Localized tooltip text for the toolbox item.
                elementGroup.CreatePrototype(), // ElementGroupPrototype (data object) representing model element on the toolbox.
                new [] { // Collection of ToolboxItemFilterAttribute objects that determine visibility of the toolbox item.
                           new System.ComponentModel.ToolboxItemFilterAttribute(ToolboxFilterString, System.ComponentModel.ToolboxItemFilterType.Require) 
                       });
            
            return item;
        }
    }
}
