using System.Linq;
using System.Text.RegularExpressions;
using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class CodeBlock
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateNames(ValidationContext context)
        {
            // all inputs and outputs must have a unique name
            // no input must have the same name as a output
            // since all will become a function argument
            var inputs = this.Inputs.Cast<CodeBlockFieldBase>();
            var outputs = this.Outputs.Cast<CodeBlockFieldBase>();

            var allFields = inputs.Concat(outputs);
            
            ValidationHelper.CheckNameUniqueness(context,
                                                 allFields,
                                                 "CodeBlock.ValidateNames",
                                                 r => r.Name);
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateFieldsCount(ValidationContext context)
        {
            if (this.Inputs.Count + this.Outputs.Count == 0)
            {
                string msg = string.Format("The CodeBlock '{0}' must have at least one Parameter or one Return Value.", this.Name);
                context.LogError(msg, "CodeBlock.ValidateFieldsCount", this);
            }
        }

        private static readonly Regex validCNameRegex = new Regex("[a-zA-Z_][a-zA-Z0-9_]*");

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCName(ValidationContext context)
        {
            if (this.Name == null || this.Name.Trim() == "")
            {
                string msg = "The Name of a Code Block must not be empty.";
                context.LogError(msg, "CodeBlock.ValidateCName", this);
            }
            else
            {
                // the name is not empty, but is es a valid code name
                Match m = validCNameRegex.Match(this.Name);
                if (m.Value != this.Name)
                {
                    string msg = string.Format("The Name '{0}' of a Code Block is not a valid identifier.", this.Name);
                    context.LogError(msg, "CodeBlock.ValidateCName", this);
                }
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateMissingInputs(ValidationContext context)
        {
            FieldedDefinitionBase.ValidateMissingInputs(context, this, this.Inputs.Cast<FieldBase>(), this.Name);
        }
    }
}
