using System.Linq;
using System.Text.RegularExpressions;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class CodeBlockFieldBase
    {
        private static readonly Regex validCNameRegex = new Regex("[a-zA-Z_][a-zA-Z0-9_]*");

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCName(ValidationContext context)
        {
            if (this.Name == null || this.Name.Trim() == "")
            {
                string parentname = this is CodeBlockInputField ? (this as CodeBlockInputField).CodeBlock.Name : (this as CodeBlockOutputField).CodeBlock.Name;
                string msg = string.Format("The Name of the Field of the Code Block '{0}' must not be empty.", parentname);
                context.LogError(msg, "CodeBlockFieldBase.ValidateCName", this);
            }
            else
            {
                // the name is not empty, but is es a valid code name
                Match m = validCNameRegex.Match(this.Name);
                if (m.Value != this.Name)
                {
                    string parentname = this is CodeBlockInputField ? (this as CodeBlockInputField).CodeBlock.Name : (this as CodeBlockOutputField).CodeBlock.Name;
                    string msg = string.Format("The Name '{1}' of a Field of the Code Block '{0}' is not a valid identifier.", parentname, this.Name );
                    context.LogError(msg, "CodeBlockFieldBase.ValidateCName", this);
                }
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateType(ValidationContext context)
        {
            if (this.Type == null || string.IsNullOrEmpty(this.Type.Value))
            {
                string msg = string.Format("The Type of the Code Block Field '{0}' must not be empty.", this.Name);
                context.LogError(msg, "CodeBlockFieldBase.ValidateType", this);
            }
            else
            {
                CodeBlock block = this is CodeBlockInputField
                                      ? (this as CodeBlockInputField).CodeBlock
                                      : (this as CodeBlockOutputField).CodeBlock;

                DataFlowModel model = block.FlowBase.GetDataFlowModel();

                if  (model != null &&
                     model.DataTypes != null &&
                    !model.DataTypes.Contains(this.Type.Value))
                {
                    string msg = string.Format("The Type ('{1}') of the Code Block Field '{0}' is not defined in the Hardware Description.", this.Name, this.Type.Value);
                    context.LogError(msg, "CodeBlockFieldBase.ValidateType", this);
                }
            }

        }
    }
}
