using System.Collections.ObjectModel;
using System.Linq;
using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class DataFlowModel
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)] 
        private void ValidateModelBasics(ValidationContext context)
        {
            if ( this.TriggerFlow == null || this.ProgramFlow == null )
                context.LogError("Trigger and Data Flow must not be null.", "DataFlowModel.ValidateModelBasics", this);
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)] 
        private void ValidateEvents(ValidationContext context)
        {
            int eventsInProgramFlow = this.ProgramFlow.FlowElements.OfType<InputEvent>().Count();

            // max one event in the program flow
            if ( eventsInProgramFlow > 1 )
            {
                string msg = "Only one event is allowed in the Data Flow.";
                context.LogError(msg, "DataFlowModel.ValidateEvents", this.ProgramFlow.FlowElements.OfType<InputEvent>().ToArray());
            }

            if ( eventsInProgramFlow == 1 )
            {
                // this exact event and only this one must exist in the trigger flow
                int eventsInTriggerFlow = this.TriggerFlow.FlowElements.OfType<InputEvent>().Count();

                if ( eventsInTriggerFlow > 1 )
                {
                    string msg = "When using events in the Data Flow only one event is allowed it the Trgger Flow.";
                    var allEvents = this.ProgramFlow.FlowElements.OfType<InputEvent>().Concat(this.TriggerFlow.FlowElements.OfType<InputEvent>());
                    context.LogError(msg, "DataFlowModel.ValidateEvents", allEvents.ToArray());                           
                }

                if ( eventsInTriggerFlow == 1 )
                {
                    // find the events
                    InputEvent triggerEvent = this.TriggerFlow.FlowElements.OfType<InputEvent>().First();
                    InputEvent programEvent = this.ProgramFlow.FlowElements.OfType<InputEvent>().First();

                    if ( !triggerEvent.Guid.Equals(programEvent.Guid) )
                    {
                        string msg = "The Event in the Trigger Flow and Data Flow must be the same Event.";
                        context.LogError(msg, "DataFlowModel.ValidateEvents", triggerEvent,programEvent);            
                    }

                }
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCodeblockNames(ValidationContext context)
        {
            ReadOnlyCollection<ModelElement> codeblocks = this.Partition.ElementDirectory.FindElements(CodeBlock.DomainClassId);

            ValidationHelper.CheckNameUniqueness(context,
                                                 codeblocks.Cast<CodeBlock>(),
                                                 "Platform.ValidateCodeblockNames",
                                                 r => r.Name);
        }
    }
}
