using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class FieldedDefinitionBase
    {
        protected int ValidateMissingInputsBase(ValidationContext context)
        {
            return ValidateMissingInputs(context, this, this.Fields.Cast<FieldBase>(), this.Name);
        }

        /// <summary>
        /// Ensures that every non optional field has an incomming connection
        /// </summary>
        /// <returns>The number of messages reported in this validation function.</returns>
        internal static int ValidateMissingInputs(ValidationContext context, FlowElement element, IEnumerable<FieldBase> fields, string name)
        {
            int msgCount = 0;

            if ( element is FunctionReturn )
                if ( (element as FunctionReturn).FunctionCall != null )
                    return msgCount;

            // order incomming connections bei guid
            HashSet<Guid> incommingConnections = new HashSet<Guid>();
            foreach (FlowElementReferencesTargetFlowElements link in FlowElementReferencesTargetFlowElements.GetLinksToSourceFlowElements(element))
                if ( !link.TargetEntry.Equals(Guid.Empty))
                    incommingConnections.Add(link.TargetEntry);

            foreach (FieldBase field in fields)
            {
                bool mustBeSet = true;
                if (field is OptionalbleField)
                    mustBeSet = !(field as OptionalbleField).Optional;

                if ( mustBeSet && !incommingConnections.Contains(field.Guid) )
                {
                    string fieldName = field is Field ? (field as Field).Name : (field as CodeBlockFieldBase).Name;
                    string msg = string.Format("The Field '{0}' of {1} '{2}' must be set by an incomming connection.", fieldName, element.GetType().Name, name);
                    context.LogError(msg, "FieldedDefinitionBase.ValidateMissingInputs", field);

                    msgCount++;
                }
            }

            return msgCount;
        }
    }
}
