using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class FunctionCall
    {        
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidatePosition(ValidationContext context)
        {
            ValidateExistenceInTriggerFlow(context, this);
        }

        internal static void ValidateExistenceInTriggerFlow(ValidationContext context, FieldedDefinitionBase element)
        {
            ProgramFlow flow = element.FlowBase as ProgramFlow;
            if ( flow != null )
            {
                if ( element.Fields.Count == 0 )
                {
                    string msg = string.Format("The empty Function '{0}' must not be present in the Data Flow, only in the Trigger Flow.", element.Name);
                    context.LogError(msg, "FunctionCall.ValidateExistenceInTriggerFlow", element);
                }

                if ( flow.DataFlowModel.TriggerFlow.FlowElements.OfType<FunctionCall>().FirstOrDefault( x => x.Guid.Equals(element.Guid) ) == null )
                {
                    string msg = string.Format("The Function '{0}' used in the Data Flow cannot be found in the Trigger Flow.", element.Name);
                    context.LogError(msg, "FunctionCall.ValidateExistenceInTriggerFlow", element);
                }
            }
        }

        internal static bool ExistFunctionCallInTriggerFlow(FunctionReturn fret)
        {
            ProgramFlow flow = fret.FlowBase as ProgramFlow;
            return flow.DataFlowModel.TriggerFlow.FlowElements.OfType<FunctionCall>().Any(x => x.Guid.Equals(fret.Guid));
        }
    }
}
