using System;
using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class InputEvent
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateOutgoingConnections(ValidationContext context)
        {
            // no outgoing connections from head AND element (for the trigger flow)
            if (this.FlowBase is TriggerFlow)
            {
                var outgoingLinks = FlowElementReferencesTargetFlowElements.GetLinksToTargetFlowElements(this);

                int fromHead = outgoingLinks.Where(x => x.SourceEntry.Equals(Guid.Empty)).Count();
                int fromEntry = outgoingLinks.Where(x => !x.SourceEntry.Equals(Guid.Empty)).Count();

                if ( fromHead > 0 && fromEntry > 0 )
                {
                    string msg = string.Format("The Event '{0}' must not have outgoing connections from both the Head and the Elements.", this.Name);
                    context.LogError(msg, "TriggerFlow.ValidateEventGraphComponents", this);
                }
            }
        }
    }
}
