using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    [ValidationState(ValidationState.Enabled)]
    partial class Operation
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateInputCount(ValidationContext context)
        {
            int min = 0;
            int max = 0;

            switch (this.Kind)
            {
                case OperationKind.True:
                case OperationKind.False:
                    min = max = 0;
                    // no input allowed
                    break;

                case OperationKind.LogicalNot:
                    min = max = 1;
                    // exactly one input needed
                    break;

                case OperationKind.LogicalXor:
                    min = max = 2;
                    // exactly two inputs needed
                    break;

                case OperationKind.LogicalAnd:
                case OperationKind.LogicalOr:
                    min = 2;
                    max = int.MaxValue;
                    // two or more inputs
                    break;
            }

            int inputs = this.SourceFlowElements.Count;

            if ( inputs < min )
            {
                string msg = string.Format("The {0} Operation needs at least {1} incomming connections.", this.ToString(), min);
                context.LogError(msg, "Operation.ValidateInputCount", this);
            }
            if (inputs > max)
            {
                string msg = string.Format("The {0} Operation must not have more than {1} incomming connections.", this.ToString(), max);
                context.LogError(msg, "Operation.ValidateInputCount", this);
            }
        }
    }
}
