using System.Collections.ObjectModel;
using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.DataFlow
{
    partial class Record
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateMissingInputs(ValidationContext context)
        {
            ReadOnlyCollection<FlowElementReferencesTargetFlowElements> incommingLinks = FlowElementReferencesTargetFlowElements.GetLinksToSourceFlowElements(this);
            bool incommingConnectionToHead = incommingLinks.Any(x => x.TargetEntry.Equals(System.Guid.Empty));
            int incommingConnectionToFields = incommingLinks.Count(x => !x.TargetEntry.Equals(System.Guid.Empty));

            if ( this.FlowBase is TriggerFlow 
                 && incommingConnectionToFields > 0 )
            {
                string msg =
                    string.Format(
                        "The Record '{0}' in the Trigger part of the Dataflow must be set by a connection to the head. Connections into the fields are not allowed.",
                        this.Name);
                context.LogError(msg, "Record.ValidateMissingInputs", this);
                return;                
            }

            if (incommingConnectionToHead)
            {
                if (incommingLinks.Count > 1)
                {
                    // there are more connection than the one to the head
                    string msg =
                        string.Format(
                            "The Record '{0}' can only have an incomming Connection to the Head or to the Fields, not both.",
                            this.Name);
                    context.LogError(msg, "Record.ValidateMissingInputs", this);
                }
            }
            else
                this.ValidateMissingInputsBase(context);

            
        }
    }
}
