using System.Linq;
using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.Datastructures
{
    [ValidationState(ValidationState.Enabled)]
    partial class DatastructureModel
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateNames(ValidationContext context)
        {
            ValidationHelper.CheckNameUniqueness(context,
                            this.Variables, 
                            "DatastructureModel.ValidateNames",
                            r => r.Name);

            ValidationHelper.CheckNameUniqueness(context,
                            this.Records, 
                            "DatastructureModel.ValidateNames",
                            r => r.Name);

            ValidationHelper.CheckNameUniqueness(context,
                            this.OutputEvents, 
                            "DatastructureModel.ValidateNames",
                            r => r.Name);

            ValidationHelper.CheckNameUniqueness(context,
                            this.Functions, 
                            "DatastructureModel.ValidateNames",
                            r => r.Name);
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateRecordIds(ValidationContext context)
        {
            ValidationHelper.CheckUniqueness(context, 
                            this.Records, "Record ID", 
                            "DatastructureModel.ValidateRecordIds",
                            r => r.RecordId,
                            r => r.Name);
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateInternalRecordIds(ValidationContext context)
        {
            var all = this.Variables.Cast<ExternalVisibleDomainClass>().Concat(
                    this.Functions.Cast<ExternalVisibleDomainClass>().Concat(
                    this.OutputEvents.Cast<ExternalVisibleDomainClass>()));

            ValidationHelper.CheckUniqueness(context,
                            all, "Internal Record ID",
                            "DatastructureModel.ValidateInternalRecordIds",
                            r => r.InternalRecordId,
                            r => r.Name);
        }

    }
}
