using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.Datastructures
{
    [ValidationState(ValidationState.Enabled)]
    partial class FieldBase
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateType(ValidationContext context)
        {
            if (this.Type == null || string.IsNullOrEmpty(this.Type.Value))
            {
                string type = this.GetType().Name;
                string msg = string.Format("The Type of the {1} '{0}' must not be empty.", this.Name, type);
                context.LogError(msg, "FieldBase.ValidateType", this);
            }
            else
            {
                DatastructureModel model = this.Partition.ElementDirectory.FindElements(DatastructureModel.DomainClassId).FirstOrDefault() as DatastructureModel;

                if (model != null &&
                    model.DataTypes != null &&
                    !model.DataTypes.Contains(this.Type.Value))
                {
                    string type = this.GetType().Name;
                    string msg = string.Format("The Type ('{1}') of the {2} '{0}' is not defined in the Hardware Description.", this.Name, this.Type.Value, type);
                    context.LogError(msg, "Variable.ValidateType", this);
                }
            }
        }
    }
}
