using System;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.Datastructures
{
    [ValidationState(ValidationState.Enabled)]
    partial class OptionalField
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateDefaultValue(ValidationContext context)
        {
            if ( !this.Optional )
                return;

            DslDataType dataType = this.Type.AsDslType(this.Store);
            switch (dataType)
            {
                case DslDataType.Boolean:
                    bool x;
                    if (!Boolean.TryParse(this.DefaultValue, out x))
                    {
                        string msg = string.Format("The Default Value ('{1}') of Field '{0}' is not a boolean value. Please use 'True' or 'False'.", this.Name, this.DefaultValue);
                        context.LogError(msg, "OptionalField.ValidateDefaultValue", this);
                    }
                    break;

                case DslDataType.Number:
                    int i;
                    if (!Int32.TryParse(this.DefaultValue, out i))
                    {
                        string msg = string.Format("The Default Value ('{1}') of Field '{0}' is not a numeric value.", this.Name, this.DefaultValue);
                        context.LogError(msg, "OptionalField.ValidateDefaultValue", this);
                    }
                    break;

                case DslDataType.String:
                    // every string can be parsed to string ;)
                    break;

                case DslDataType.Record:
                case DslDataType.Other:
                    string msg2 = string.Format("The Field '{0}' cannot be optional, since of its Data Type.", this.Name);
                    context.LogError(msg2, "OptionalField.ValidateDefaultValue", this);
                    break;

                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

    }
}
