using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using BenjaminSchroeter.Dsl.CompartmentMapping;
using Microsoft.VisualStudio.Modeling.Shell;

namespace BenjaminSchroeter.Flow.Datastructures
{
    partial class DatastructuresCommandSet
    {
        private const string guidDatastructuresString = "BB185FF7-35A1-4abf-9BD3-9FBC01CA1E2D";
        public static readonly Guid guidDatastructures = new Guid(guidDatastructuresString);

        public const uint cmdIdShowExplorer = 0x2020;

        protected override IList<MenuCommand> GetMenuCommands()
        {
            IList<MenuCommand> commands = base.GetMenuCommands();

            // remove some commands
            CompartmentMappingUtil.RemoveCompartmentShapeExpandCollapseCommand(commands);

            // and add some
            commands.Add(CreateShowExplorerCommand());

            return commands;
        }

        /// <summary>
        /// Creates the "show datastructure explorer" command.
        /// </summary>
        /// <remarks>
        /// On the context menu of Datastrucures model pane.
        /// </remarks>        
        /// <returns></returns>
        private MenuCommand CreateShowExplorerCommand()
        {
            CommandID menuCommandID = new CommandID(guidDatastructures, (int)cmdIdShowExplorer);

            DynamicStatusMenuCommand menuItem = new DynamicStatusMenuCommand(
                OnStatusViewModelExplorer,
                OnMenuViewModelExplorer,
                menuCommandID); 

            return menuItem;
        }
        /// <summary>
        /// Shows the "show datastructre explorer" command only if no shape is selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnStatusViewModelExplorer(object sender, EventArgs e)
        {
            MenuCommand cmd = sender as MenuCommand;

            bool show = this.CurrentDocumentSelection.Count > 0;
            foreach (object o in this.CurrentDocumentSelection)
            {
                if ( !(o is DatastructuresDiagram) )
                {
                    show = false;
                    break;
                }
            }

            cmd.Enabled = cmd.Visible = show;
        }
    }
}
