using System.Diagnostics;
using System.Linq;
using BenjaminSchroeter.Flow.DataTypes;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using BenjaminSchroeter.Flow.HardwareDescription.DataService;
using BenjaminSchroeter.VsxTools;
using Microsoft.VisualStudio.Shell.Interop;

namespace BenjaminSchroeter.Flow.Datastructures
{
    partial class DatastructuresDocView
    {
        protected override bool LoadView()
        {
            // when the diagram is loaded the DataTypes property of the model
            // should be initialized from the HardwareDescription of the same modek

            bool open = base.LoadView();
            if (open)
            {
                // get the HierarchyItem of this diagram
                HierarchyItem hierarchyItem = GetHierarchyItem();

                // get the data service
                IHardwareDescriptionDataService dataservice = this.GetService(typeof(SHardwareDescriptionDataService)) as IHardwareDescriptionDataService;
                Debug.Assert(dataservice != null, "IHardwareDescriptionDataService");

                try
                {
                    HardwareDescriptionModelData hardware = dataservice.GetHardwareDescription(hierarchyItem.VsHierarchy);

                    foreach (DataTypeData data in hardware.DataTypes)
                        DataTypeConverter.AddDataType(this.DocData.Store, data.Name, data.CType, data.DslType);
                    
                    DatastructureModel dataflow = (DatastructureModel)this.DocData.RootElement;
                    dataflow.DataTypes = hardware.DataTypes.Select(x => x.Name).ToArray();
                }
                catch (WrongNumberOfHardwareDescriptionException) {}
            }

            return open;
        }

        private HierarchyItem GetHierarchyItem()
        {
            IVsRunningDocumentTable rdt = this.GetService(typeof(IVsRunningDocumentTable)) as IVsRunningDocumentTable;
            Debug.Assert(rdt != null, "IVsRunningDocumentTable");

            HierarchyItem hierarchyItem = null;
            foreach (IVsRunningDocumentTableExtensions.RdtInfo info in rdt.GetDocDataForOpenDocuments())
                if (info.DocData == this.DocData)
                {
                    hierarchyItem = info.Item;
                    break;
                }

            return hierarchyItem;
        }
    }
}
