using System.ComponentModel;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.Flow.DataTypes
{
    /// <summary>
    /// This class is used in all DSL models as the Type property.
    /// 
    /// Internaly a simple string is stored.
    /// </summary>
    [TypeConverter("BenjaminSchroeter.Flow.DataTypes.DataTypeConverter")]
    public class DataTypeString
    {
        /// <summary>
        /// Gets or sets the actual value.
        /// </summary>
        /// <value>The value.</value>
        public string Value { get; set; }

        public DataTypeString()
        {
            Value = "";
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DataTypeString"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public DataTypeString(string value)
        {
            this.Value = value;
        }

        public override string ToString()
        {
            return Value;
        }

        /// <summary>
        /// Returns the C type of this data type using the given store to translate name to c type.
        /// </summary>
        /// <param name="lookInStore">The look in store.</param>
        /// <returns></returns>
        public string AsCType(Store lookInStore)
        {
            return DataTypeConverter.GetDataTypeCName(lookInStore, this.Value);
        }

        /// <summary>
        /// Returns the DSL type of this data type using the given store to translate name to dsl type.
        /// </summary>
        /// <param name="lookInStore">The look in store.</param>
        /// <returns></returns>
        public DslDataType AsDslType(Store lookInStore)
        {
            return DataTypeConverter.GetDataTypeDslType(lookInStore, this.Value);
        }

        /// <summary>
        /// Clones this instance.
        /// </summary>
        /// <returns></returns>
        public DataTypeString Clone()
        {
            return new DataTypeString(this.Value);
        }

        public override bool Equals(object obj)
        {
            DataTypeString s = obj as DataTypeString;
            if ( s!= null)
                return Value.Equals(s.Value);
            return base.Equals(obj);
        }

        public override int GetHashCode()
        {
            return (Value != null ? Value.GetHashCode() : 0);
        }

        public static bool operator ==(DataTypeString left, DataTypeString right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(DataTypeString left, DataTypeString right)
        {
            return !Equals(left, right);
        }
    }
}