using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    /// <summary>
    /// This class is used to define the compatibility of a driver for a hardware platform.
    /// Every driver has one of these data structures as a property.
    /// </summary>
    [TypeConverter("BenjaminSchroeter.Flow.HardwareDescription.DriverCompatibilityTypeConverter")]
    public class DriverCompatibility
    {
        public struct EnableableRelation
        {
            public Guid DriverElement { get; set; }
            public Guid HardwareElement { get; set; }
        }

        /// <summary>
        /// Gets or sets the compatible hardware.
        /// </summary>
        /// <remarks>This is a list of guid with all guids of hardwareplatforms that can be load this driver.</remarks>
        /// <value>The compatible hardware.</value>
        public List<Guid> CompatibleHardware { get; private set; }

        /// <summary>
        /// Gets or sets the enableable relations to be added to the hardwareplatform when this driver is loaded.
        /// </summary>
        /// <value>The enableable relations.</value>
        public List<EnableableRelation> EnableableRelations { get; private set;}

        /// <summary>
        /// Initializes a new instance of the <see cref="DriverCompatibility"/> class.
        /// </summary>
        public DriverCompatibility()
        {
            CompatibleHardware = new List<Guid>();
            EnableableRelations = new List<EnableableRelation>();
        }
    }
}
