using System.Collections.Generic;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.Datastructures;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.GlobalValidation
{
    /// <summary>
    /// Base class for all global Validation rules
    /// </summary>
    abstract class GlobalValidationRule
    {
        /// <summary>
        /// File name of the currently validating project.
        /// </summary>
        protected readonly string projectFileName;
        private readonly GlobalValidationErrorListObserver errorListObserver;

        /// <summary>
        /// Initializes a new instance of the <see cref="GlobalValidationRule"/> class.
        /// </summary>
        /// <param name="projectFileName">Name of the project file.</param>
        /// <param name="errorListObserver">The error list observer.</param>
        protected GlobalValidationRule(string projectFileName, GlobalValidationErrorListObserver errorListObserver)
        {
            this.projectFileName = projectFileName;
            this.errorListObserver = errorListObserver;
        }

        /// <summary>
        /// Validates the specified models.
        /// </summary>
        /// <param name="hardwareInfos">The hardware descriptions.</param>
        /// <param name="datastructures">The datastructures.</param>
        /// <param name="dataflows">The dataflows.</param>
        public abstract void Validate(Dictionary<string, HardwareDescriptionModel> hardwareInfos, Dictionary<string, DatastructureModel> datastructures, Dictionary<string, DataFlowModel> dataflows);

        /// <summary>
        /// Logs an error.
        /// </summary>
        protected void LogError(string filename, string message, params object[] args)
        {
            string m = string.Format(message, args);
            errorListObserver.AddMessage(m, filename);
        }

        /// <summary>
        /// Logs an error for the project file.
        /// </summary>
        protected void LogProjectError(string message, params object[] args)
        {
            this.LogError(projectFileName, message, args);
        }
    }
}
