using System.Collections.Generic;
using System.IO;
using System.Linq;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.Datastructures;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.GlobalValidation
{
    /// <summary>
    /// Global Validation Rule to ensure that every Dataflow has a unique filename, even if they are placed in different folders.
    /// </summary>
    class DataFlowFilenameUniqueness : GlobalValidationRule
    {
        public DataFlowFilenameUniqueness(string projectFileName, GlobalValidationErrorListObserver errorListObserver) : base(projectFileName, errorListObserver)
        {
        }

        public override void Validate(Dictionary<string, HardwareDescriptionModel> hardwareInfos, Dictionary<string, DatastructureModel> datastructures, Dictionary<string, DataFlowModel> dataflows)
        {
            var groupedFilenames = from f in dataflows.Keys
                                   group f by Path.GetFileName(f);

            foreach (IGrouping<string, string> group in groupedFilenames.Where(g => g.Count() > 1))
            {
                string allFiles = string.Join(", ", group.ToArray());
                LogProjectError("The project contains more than one file with the name '{0}': {1}", group.Key, allFiles);
            }
        }
    }
}
