using System.Collections.Generic;
using System.IO;
using System.Linq;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.Datastructures;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.GlobalValidation
{
    /// <summary>
    /// Global Validation Rule to ensure that the project contains exactly on .hardwareinfo file.
    /// </summary>
    class ExactlyOneHardwareInfo : GlobalValidationRule
    {
        public ExactlyOneHardwareInfo(string projectFileName, GlobalValidationErrorListObserver errorListObserver) : base(projectFileName, errorListObserver) { }

        public override void Validate(Dictionary<string, HardwareDescriptionModel> hardwareInfos, Dictionary<string, DatastructureModel> datastructures, Dictionary<string, DataFlowModel> dataflows)
        {
            // more than onen hardware info?
            int hardwareInfoCount = hardwareInfos.Count();
            if (hardwareInfoCount == 0)
                LogProjectError("The project '{0}' does not contain any Hardwaredescription file (.hardwareinfo).", Path.GetFileName(projectFileName));
            else if (hardwareInfoCount > 1)
            {
                string files = string.Join(", ", hardwareInfos.Select(x => Path.GetFileName(x.Key)).ToArray());
                LogProjectError("The project '{0}' contains more than one Hardwaredescription file (.hardwareinfo): {1}.", Path.GetFileName(projectFileName), files);
            }
        }
    }
}
