using System.Collections.Generic;
using System.IO;
using BenjaminSchroeter.Flow.DataFlow;
using BenjaminSchroeter.Flow.Datastructures;
using BenjaminSchroeter.Flow.HardwareDescription;

namespace BenjaminSchroeter.GlobalValidation
{
    /// <summary>
    /// Global Validation Rule to ensure that every model file realy exists.
    /// </summary>
    class FileExist : GlobalValidationRule
    {
        public FileExist(string projectFileName, GlobalValidationErrorListObserver errorListObserver) : base(projectFileName, errorListObserver) { }

        public override void Validate(Dictionary<string, HardwareDescriptionModel> hardwareInfos, Dictionary<string, DatastructureModel> datastructures, Dictionary<string, DataFlowModel> dataflows)
        {
            // file exist check  
            CheckFileExists(hardwareInfos.Keys);
            CheckFileExists(datastructures.Keys);
            CheckFileExists(dataflows.Keys);
        }

        private void CheckFileExists(IEnumerable<string> files)
        {
            foreach (string file in files)
                if (!File.Exists(file))
                    LogError(file, "The file '{0}' does not exist.", file);
        }

    }
}
