<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">	
	
	<xsl:output method="xml" version="1.0" indent="yes" standalone="no" encoding="UTF-8" />

	<xsl:param name="directory" />

	<xsl:template match="/">
		<xsl:element name="CodeDescription">
			<!-- Load all doxgen generated xml files -->
			<xsl:for-each select="doxygenindex/compound">
				<xsl:apply-templates select="document(  concat($directory, @refid, '.xml' ) )/doxygen" mode="attributes"></xsl:apply-templates>
			</xsl:for-each>
		
			<xsl:for-each select="doxygenindex/compound">
				<xsl:apply-templates select="document(  concat($directory, @refid, '.xml' ) )/doxygen" mode="elements"></xsl:apply-templates>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>


	<xsl:template match="doxygen" mode="attributes">		
		<xsl:if test="compounddef/detaileddescription/para/firmwarename">
			<xsl:attribute name="firmwarename">
				<xsl:value-of select="compounddef/detaileddescription/para/firmwarename"/>
			</xsl:attribute>			
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/drivername">
			<xsl:attribute name="drivername">
				<xsl:value-of select="compounddef/detaileddescription/para/drivername"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/version">
			<xsl:attribute name="version">
				<xsl:value-of select="compounddef/detaileddescription/para/version"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/productname">
			<xsl:attribute name="productname">
				<xsl:value-of select="compounddef/detaileddescription/para/productname"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/vendorname">
			<xsl:attribute name="vendorname">
				<xsl:value-of select="compounddef/detaileddescription/para/vendorname"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/producturl">
			<xsl:attribute name="producturl">
				<xsl:value-of select="compounddef/detaileddescription/para/producturl"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/vendorurl">
			<xsl:attribute name="vendorurl">
				<xsl:value-of select="compounddef/detaileddescription/para/vendorurl"/>
			</xsl:attribute>
		</xsl:if>
		<xsl:if test="compounddef/detaileddescription/para/description">
			<xsl:attribute name="description">
				<xsl:value-of select="compounddef/detaileddescription/para/description"/>
			</xsl:attribute>
		</xsl:if>
	</xsl:template>

	<xsl:template match="doxygen" mode="elements">			
		<xsl:for-each select="compounddef/sectiondef/memberdef">
			<xsl:element name="function">
				<xsl:attribute name="name">
					<xsl:value-of select="name"/>
				</xsl:attribute>
				<xsl:attribute name="type">
					<xsl:value-of select="type"/>
				</xsl:attribute>
				<xsl:apply-templates select="briefdescription"></xsl:apply-templates>
				<xsl:apply-templates select="detaileddescription"></xsl:apply-templates>
				<xsl:apply-templates select="param"></xsl:apply-templates>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="briefdescription">
		<xsl:apply-templates select="para"></xsl:apply-templates>
	</xsl:template>
	<xsl:template match="detaileddescription">
		<xsl:apply-templates select="para"></xsl:apply-templates>
	</xsl:template>

	<xsl:template match="param">
		<xsl:element name="param">
			<xsl:attribute name="declname">
				<xsl:value-of select="declname"/>
			</xsl:attribute>
			<xsl:variable name="paraname">
				<xsl:value-of select="declname"/>
			</xsl:variable>

			<xsl:attribute name="type">
				<xsl:value-of select="type"/>
			</xsl:attribute>

			<xsl:apply-templates select="briefdescription/para"/>

			<xsl:for-each select="../detaileddescription/para/parameterlist/parameteritem/parameternamelist/parametername">
				<xsl:if test=".=$paraname">
					<xsl:apply-templates select="../../parameterdescription/para"/>
				</xsl:if>
			</xsl:for-each>

		</xsl:element>
	</xsl:template>

	<xsl:template match="para">
		<xsl:copy-of select="getVariable"/>
		<xsl:copy-of select="setVariable"/>
		<xsl:copy-of select="input"/>
		<xsl:copy-of select="inputInit"/>
		<xsl:copy-of select="output"/>
		<xsl:copy-of select="outputInit"/>
		<xsl:copy-of select="event"/>
		<xsl:copy-of select="eventCallback"/>

		<xsl:copy-of select="field"/>
		<xsl:copy-of select="fieldenabled"/>
		<xsl:copy-of select="parameter"/>

		<xsl:copy-of select="default"/>
		<xsl:copy-of select="description" />
		<xsl:copy-of select="multiplicity"/>
		<xsl:copy-of select="enabled"/>		
		<xsl:copy-of select="optional" />
	</xsl:template>	
</xsl:stylesheet>
