using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that each function contains a valid annotation.
    /// </summary>
    class Chk001 : SemanticCheckBase
    {
        public Chk001(IList<ParsingMessage> messages) : base(messages)    {}

        public override void Check(CodeDescription doc)
        {
            var x = from f in doc.function where f.ItemElementName == ItemChoiceType.none select f;
            foreach (var f in x)
                ReportError("001a","The function {0}() is missing one of the following mandatory annotations: @getVariable, @setVariable, @input, @inputInit, @output, @outputInit, @event, @eventCallback.",f.name);

            var y = from f in doc.function where f.ItemElementName != ItemChoiceType.none && (f.Item ?? "").Trim() == "" select f;
            foreach (var f in y)
                ReportError("001b", "At function {0}() the @{1} annotation is missing a name.", f.name, f.ItemElementName.ToString());
        }
    }
}
