using System.Collections.Generic;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that every firmware contains
    /// exactly one of @firmwarename and @drivername
    /// </summary>
    class Chk003_004 : SemanticCheckBase
    {
        public Chk003_004(IList<ParsingMessage> messages) : base(messages)    {}

        public override void Check(CodeDescription doc)
        {
            bool driver = (doc.drivername ?? "").Trim() != "";
            bool firmware = (doc.firmwarename ?? "").Trim() != "";

            if ( !driver && !firmware)
                ReportError("003","The file annotation @firmwarename or @drivername is missing.");

            if ( driver && firmware)
                ReportError("004", "Do not use both @firmwarename and @drivername in one file.");
        }
    }
}
