using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that @field is
    /// only used where it is allowed.
    /// </summary>
    class Chk005 : SemanticCheckBase
    {
        public Chk005(IList<ParsingMessage> messages) : base(messages)    {}

        public override void Check(CodeDescription doc)
        {
            IEnumerable<function> x = from f in doc.function
                    where
                        (f.ItemElementName == ItemChoiceType.@event
                         || f.ItemElementName == ItemChoiceType.eventCallback
                         || f.ItemElementName == ItemChoiceType.getVariable
                         || f.ItemElementName == ItemChoiceType.setVariable
                         || f.ItemElementName == ItemChoiceType.outputInit
                         || f.ItemElementName == ItemChoiceType.inputInit) && f.field != null
                    select f;

            foreach (function f in x)
                ReportError("005","The annotation @field should not be used with the @{0} function {1}().",f.ItemElementName.ToString(),f.name);
        }
    }
}
