using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that annotations
    /// only used in places where they are allowed.
    /// </summary>
    class Chk008 : SemanticCheckBase
    {
        public Chk008(IList<ParsingMessage> messages) : base(messages)    {}

        public override void Check(CodeDescription doc)
        {
            IEnumerable<function> x = from f in doc.function
                    where
                        (f.ItemElementName == ItemChoiceType.getVariable || f.ItemElementName == ItemChoiceType.setVariable)
                        && ( !string.IsNullOrEmpty(f.field) || f.multiplicitySpecified )
                    select f;

            foreach (function f in x)
                ReportError("008", "No additional annotation for {0} function {1}() necessary!", f.ItemElementName.ToString(), f.name);
        }
    }
}
