using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that the signiture of
    /// the getter and setter for Variables are well formed.
    /// </summary>
    class Chk020 : SemanticCheckBase
    {
        public Chk020(IList<ParsingMessage> messages) : base(messages) { }

        public override void Check(CodeDescription doc)
        {
            IEnumerable<function> getVariables = from f in doc.function
                                      where f.ItemElementName == ItemChoiceType.getVariable || f.ItemElementName == ItemChoiceType.setVariable
                                      select f;

            foreach (function f in getVariables)
            {
                if (f.type != "bool" || f.param == null || f.param.Length != 1)
                    ReportMsg(f);
                else
                {
                    if (f.ItemElementName == ItemChoiceType.getVariable && IsValue(f.param[0].type))
                        ReportMsg(f);

                    if (f.ItemElementName == ItemChoiceType.setVariable && IsPointer(f.param[0].type))
                        ReportMsg(f);
                }
            }
        }

        private void ReportMsg(function f)
        {
            if (f.ItemElementName  == ItemChoiceType.getVariable)
                ReportError("020a", "The signature of @getVariable function {0}() is wrong. It should be: bool {0}(T*) for some type T.", f.name);

            if (f.ItemElementName == ItemChoiceType.setVariable)
                ReportError("020b", "The signature of @setVariable function {0}() is wrong. It should be: bool {0}(T) for some type T.", f.name);
        }
    }
}
