using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that the signiture of a
    /// @event function is well formed.
    /// </summary>
    class Chk021 : SemanticCheckBase
    {
        public Chk021(IList<ParsingMessage> messages) : base(messages) { }

        public override void Check(CodeDescription doc)
        {
            IEnumerable<function> events = from f in doc.function
                                      where f.ItemElementName == ItemChoiceType.@event
                                      select f;

            foreach (function f in events)
            {
                if (f.param == null || f.param.Length < 1)
                {
                    ReportMsg(f);
                    continue;
                }

                param lastParam = f.param[f.param.Length - 1];

                if ( lastParam.@default != null ||
                    lastParam.description != null ||
                    lastParam.ItemElementName != ItemChoiceType1.none ||
                    lastParam.type != "void *" )
                    ReportMsg(f);
            }
        }

        private void ReportMsg(function f)
        {
            ReportError("021", "The last parameter of the @event function {0}() must be a function pointer 'void*' without any further annotations.", f.name);
        }
    }
}
