using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that the datatypes
    /// of function parameters are  .
    /// </summary>
    class Chk022 : SemanticCheckBase
    {
        public Chk022(IList<ParsingMessage> messages) : base(messages) { }

        public override void Check(CodeDescription doc)
        {
            var parameter = from f in doc.function from p in (f.param ?? new param[0]) select new { Parameter = p, Function = f };

            foreach (var p in parameter)
            {
                if ( (p.Parameter.ItemElementName == ItemChoiceType1.enabled || p.Parameter.ItemElementName == ItemChoiceType1.fieldenabled)
                            && p.Parameter.type != "bool" )
                    ReportError("022a","The type of parameter '{0}' in function {1}() must be 'bool'.",p.Parameter.declname,p.Function.name);

                if ( p.Parameter.ItemElementName == ItemChoiceType1.parameter && IsPointer(p.Parameter.type) )
                    ReportError("022b", "The type of parameter '{0}' in function {1}() must not be a pointer.", p.Parameter.declname, p.Function.name);

                if (p.Parameter.ItemElementName == ItemChoiceType1.field &&
                    p.Function.ItemElementName == ItemChoiceType.input &&
                    !IsPointer(p.Parameter.type))
                    ReportError("022c", "The type of parameter '{0}' in function {1}() must be a pointer.", p.Parameter.declname, p.Function.name);
            }
        }
    }
}
