using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that input and output
    /// function for single fileds only contain a single parameter.
    /// </summary>
    class Chk023 : SemanticCheckBase
    {
        public Chk023(IList<ParsingMessage> messages) : base(messages) { }

        public override void Check(CodeDescription doc)
        {
            var funct = from f in doc.function
                                      where (f.ItemElementName == ItemChoiceType.input || f.ItemElementName == ItemChoiceType.output)
                                                && !string.IsNullOrEmpty(f.field)
                                      select f;

            foreach (function f in funct)
            {
                if (f.param == null || f.param.Length != 1)
                    ReportError("023","The function {0}() must have exactly one parameter.", f.name);
            }
        }
    }
}
