using System;
using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that function parameters
    /// only contain allowed annotations 
    /// </summary>
    class Chk050 : SemanticCheckBase
    {
        Dictionary<ItemChoiceType, ItemChoiceType1[]> allowedAnnotations = new Dictionary<ItemChoiceType, ItemChoiceType1[]>();

        public Chk050(IList<ParsingMessage> messages): base(messages)
        {
            allowedAnnotations.Add(ItemChoiceType.eventCallback, new[] { ItemChoiceType1.field });
            allowedAnnotations.Add(ItemChoiceType.input, new[] { ItemChoiceType1.field });
            allowedAnnotations.Add(ItemChoiceType.output, new[] { ItemChoiceType1.field });

            allowedAnnotations.Add(ItemChoiceType.inputInit, new[] { ItemChoiceType1.parameter, ItemChoiceType1.enabled, ItemChoiceType1.fieldenabled });
            allowedAnnotations.Add(ItemChoiceType.outputInit, new[] { ItemChoiceType1.parameter, ItemChoiceType1.enabled, ItemChoiceType1.fieldenabled });
            allowedAnnotations.Add(ItemChoiceType.@event, new[] { ItemChoiceType1.parameter, ItemChoiceType1.enabled });

            allowedAnnotations.Add(ItemChoiceType.getVariable, new[] { ItemChoiceType1.none  });
            allowedAnnotations.Add(ItemChoiceType.setVariable, new[] { ItemChoiceType1.none });
        }


        public override void Check(CodeDescription doc)
        {
            foreach (function f in doc.function)
            {
                if ( f.param != null)
                {
                    ItemChoiceType1[] myAllowedAnnotations = allowedAnnotations[f.ItemElementName];

                    if ( (f.ItemElementName == ItemChoiceType.input || f.ItemElementName == ItemChoiceType.output) && !string.IsNullOrEmpty(f.field) )
                            myAllowedAnnotations = new[] { ItemChoiceType1.none};

                    param[] checkParams;
                    if (f.ItemElementName == ItemChoiceType.@event && f.param.Length >= 1)
                    {
                        checkParams = new param[f.param.Length-1];
                        Array.Copy(f.param,checkParams,f.param.Length-1);
                    }
                    else
                        checkParams = f.param;

                    foreach (param p in checkParams)
                    {
                        if ( !myAllowedAnnotations.Contains(p.ItemElementName) )
                        {
                            if ( myAllowedAnnotations.Contains(ItemChoiceType1.none))
                            {
                                ReportError("050a", "The parameter {0} of function {1}() must not have any annotations.", p.declname, f.name);
                            }
                            else
                            {
                                List<string> l = new List<string>();
                                foreach (ItemChoiceType1 type in myAllowedAnnotations)
                                    l.Add(type.ToString());
                                string str = string.Join(", ", l.ToArray());
                                ReportError("050b","The parameter {0} of function {1}() is missing one of the following mandatory annotations: {2}.",p.declname, f.name, str);
                            }
                        }
                    }
                }
            }
        }

       
    }
}
