using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.Firmwareparser.SemanticChecks
{
    /// <summary>
    /// Semantic Check Rule to ensure that @parametes do not have a @default annotation.
    /// </summary>
    class Chk051 : SemanticCheckBase
    {
        public Chk051(IList<ParsingMessage> messages) : base(messages)    {}

        public override void Check(CodeDescription doc)
        {
            var parameter = from f in doc.function from p in (f.param ?? new param[0]) select new {Parameter = p, Function = f};

            var wrongDefault = from p in parameter
                               where
                                   p.Parameter.ItemElementName != ItemChoiceType1.parameter &&
                                   p.Parameter.@default != null
                               select p;
            foreach (var p in wrongDefault)
                ReportError("051","The parameter {0} of {1}() must not have a @default annotation.",p.Parameter.declname, p.Function.name );
        }
    }


}
