using System;
using System.Collections.Generic;
using System.Linq;

namespace BenjaminSchroeter.Flow.HardwareDescription.Data
{
    partial class HardwareDescriptionModelData
    {
        /// <summary>
        /// Finds the variable with a given GUID.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <returns></returns>
        public VariableData FindVariable(Guid guid)
        {
            return FindSomething(guid, p => p.Variables);
        }

        /// <summary>
        /// Finds the input with a given GUID.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <returns></returns>
        public InputData FindInput(Guid guid)
        {
            return FindSomething(guid, p => p.Inputs);
        }

        /// <summary>
        /// Finds the event with a given GUID.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <returns></returns>
        public EventData FindEvent(Guid guid)
        {
            return FindSomething(guid, p => p.Events);
        }

        /// <summary>
        /// Finds the output with a given GUID.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <returns></returns>
        public OutputData FindOutput(Guid guid)
        {
            return FindSomething(guid, p => p.Outputs);
        }

        private T FindSomething<T>(Guid guid, Func<PlatformData, IList<T>> getList) where T : EnableableDomainClassData
        {
            foreach (PlatformData platform in this.Platforms)
            {
                T found = getList(platform).FirstOrDefault(x => x.Guid.Equals(guid));

                if (found != null)
                    return found;
            }

            return null;
        }
    }
}