﻿using System;

namespace BenjaminSchroeter.Dsl.UserRestrictions
{
    /// <summary>
    /// This attribute decorates a Diagram class 
    /// and references a ToolboxItem (by Id and TabNameId).
    /// 
    /// If the restriction mode matches this ToolboxItem will not be shown.
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true)]
    public class RestrictToolboxItemUsingAttribute : Attribute
    {
        /// <summary>
        /// Gets or sets the restriction mode this setting is used for.
        /// </summary>
        /// <value>The restriction mode this setting is used for.</value>
        public int RestrictionMode { get; set; }

        /// <summary>
        /// Gets or sets the item id of the ToolboxItem.
        /// </summary>
        /// <value>The item id of the ToolboxItem.</value>
        
        public string ItemId { get; set; }
        /// <summary>
        /// Gets or sets the tab name id of the ToolboxItem.
        /// </summary>
        /// <value>The tab name id of the ToolboxItem.</value>
        public string TabNameId { get; set; }


        /// <summary>
        /// Initializes a new instance of the <see cref="RestrictToolboxItemUsingAttribute"/> class.
        /// </summary>
        /// <param name="restrictionMode">The restriction mode.</param>
        /// <param name="itemId">The item id of the ToolboxItem.</param>
        /// <param name="tabNameId">The tab name id of the ToolboxItem.</param>
        public RestrictToolboxItemUsingAttribute(int restrictionMode, string itemId, string tabNameId)
        {
            ItemId = itemId;
            RestrictionMode = restrictionMode;
            TabNameId = tabNameId;
        }
    }
}
