using System.Collections.Generic;
using BenjaminSchroeter.Dsl.UserRestrictions;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    partial class HardwareDescriptionSerializationHelper
    {
        /// <summary>
        /// Helper method to create and initialize a new HardwareDescriptionDiagram.
        /// </summary>
        /// <param name="diagramPartition"></param>
        /// <param name="modelRoot"></param>
        /// <returns></returns>
        protected internal override HardwareDescriptionDiagram CreateDiagramHelper(Microsoft.VisualStudio.Modeling.Partition diagramPartition, Microsoft.VisualStudio.Modeling.ModelElement modelRoot)
        {
            // When a new model and diagram is created it contains by default 
            // one Hardware- and one DriverPlatform.
            // But I can only use one of them (based on the restioction mode.
            HardwareDescriptionDiagram diagram = base.CreateDiagramHelper(diagramPartition, modelRoot);
            HardwareDescriptionModel model = (HardwareDescriptionModel) modelRoot;

            int currentMode = UserRestrictionProvider.GetRestrictionMode(model.Store);



            // find all platforms to delete
            List<Platform> toDelete = new List<Platform>();
            foreach (Platform platform in model.Platforms)
            {
                // driver platforms on hardware models
                if (    (currentMode == (int) RestrictionMode.Hardware || currentMode == (int) RestrictionMode.HardwareInfo)
                     &&  platform is DriverPlatform )
                    toDelete.Add(platform);
                // hardware platforms on driver models
                else if ((currentMode == (int)RestrictionMode.Driver || currentMode == (int)RestrictionMode.DriverInfo)
                     && platform is HardwarePlatform)
                    toDelete.Add(platform);
            }
            
            // and delete them
            foreach (Platform platform in toDelete)
                model.Platforms.Remove(platform);

            return diagram;
        }
    }
}
