using System.Drawing;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    partial class PortShape
    {
        /// <summary>
        /// Shows a icons for disabled fields.
        /// </summary>
        /// <remarks>
        /// In fact enabled and disabled fields will have an icon, but the enabled icon is invisible.
        /// </remarks>
        /// <param name="mappings"></param>
        /// <returns></returns>
        protected CompartmentMapping[] ShowIconsForEnableableFileds(CompartmentMapping[] mappings)
        {
            foreach (CompartmentMapping mapping in mappings)
            {
                ElementListCompartmentMapping compMapping = mapping as ElementListCompartmentMapping;
                if (compMapping != null)
                {
                    compMapping.ImageGetter = ListItemDisplayImageGetterForEnableableFields;
                    
                    // this one can be used to change the text of the item as well
                    // compMapping.StringGetter = ListItemDisplayStringGetterForEnableableFields;
                }
            }

            // return the changed list
            return mappings;
        }

        private string ListItemDisplayStringGetterForEnableableFields(ModelElement element)
        {
            if (!(element is EnableableFieldBase))
                return element.ToString();

            EnableableFieldBase field = (EnableableFieldBase)element;
            return field.Enabled ? field.Name : "(" + field.Name + ")";
        }

        private Image ListItemDisplayImageGetterForEnableableFields(ModelElement element)
        {
            if (!(element is EnableableFieldBase))
                return Resource.FieldEnabled;

            return ((EnableableFieldBase)element).Enabled ? Resource.FieldEnabled : Resource.FieldDisabled;
        }
    }
}
