using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    /// <summary>
    /// When the enabled flag changes the background color must be change, too.
    /// If the element is a field, the icon left to the field should be updated
    /// </summary>
    [RuleOn(typeof(EnableableDomainClass), FireTime = TimeToFire.LocalCommit)]
    class EnabledChangeColorChangeRule : ChangeRule
    {
        public override void ElementPropertyChanged(ElementPropertyChangedEventArgs e)
        {
            // only for the enabled property
            if (e.DomainProperty.Id.Equals(EnableableDomainClass.EnabledDomainPropertyId))
            {
                // find the shape for this model element
                ShapeElement shape = e.ModelElement.GetFirstShape();

                // update color of variable and port shapes
                VariableShape vs = shape as VariableShape;
                if ( vs != null )
                    vs.SetBackgroundColor((bool) e.NewValue);
                PortShape ps = shape as PortShape;
                if (ps != null)
                    ps.SetBackgroundColor((bool)e.NewValue);


                // this is code for immediately update the icon next to a disabled field in a compartment shape 
                if (e.ModelElement is InputField)
                {
                    Input x = (e.ModelElement as InputField).Input;
                    ShapeElement s = x.GetFirstShape();
                    if (s != null)
                        s.Invalidate();
                }
                if (e.ModelElement is OutputField)
                {
                    Output x = (e.ModelElement as OutputField).Output;
                    ShapeElement s = x.GetFirstShape();
                    if (s != null)
                        s.Invalidate();
                }
            }
        }
    }
}
