using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    partial class PortShape
    {
        protected override void InitializeInstanceResources()
        {
            base.InitializeInstanceResources();

            // update the background color when creating this element
            Port port = (this.ModelElement as Port);
            if (port != null)
                SetBackgroundColor(port.Enabled);
        }

        /// <summary>
        /// Sets the background color based on the enabled flag.
        /// </summary>
        /// <param name="enabled"></param>
        internal void SetBackgroundColor(bool enabled)
        {
            if (enabled)
            {
                // get the original brush
                BrushSettings enabledBrushSetting = this.ClassStyleSet.GetOverriddenBrushSettings(DiagramBrushes.ShapeBackground);
                this.StyleSet.OverrideBrush(DiagramBrushes.ShapeBackground, enabledBrushSetting);
            }
            else
            {
                // create a new brush with the gray background color
                BrushSettings brushSettings = new BrushSettings {Color = HardwareDescriptionConstants.DISABLED_SHAPE_BACKGROUND_COLOR};
                this.StyleSet.OverrideBrush(DiagramBrushes.ShapeBackground, brushSettings);
            }

            // paint the shape with the new color 
            this.Invalidate();
        }
    }
}
