using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    /// <summary>
    /// This Class is used to create the Datatypes.
    /// 
    /// It can be used to keep the data form an existing object
    /// </summary>
    internal class DataTypeCreator
    {
        private Dictionary<string, DataType> cache;

        /// <summary>
        /// Initializes a new instance of the <see cref="DataTypeCreator"/> class.
        /// </summary>
        public DataTypeCreator()
        {
            cache = new Dictionary<string, DataType>();
        }

        /// <summary>
        /// Inits the data types from existing model.
        /// </summary>
        /// <param name="model">The model.</param>
        public void InitDataTypesFromModel(HardwareDescriptionModel model)
        {
            foreach (DataType type in model.DataTypes)
                this.cache.Add(type.CType, type);
        }

        /// <summary>
        /// Creates a datatype for the given c type.
        /// </summary>
        /// <param name="cType">The C Type used in the source code.</param>
        /// <param name="store">The store to create the new Datatype in.</param>
        /// <param name="Name">The name for the new type. The name must be changed AFTER adding it to the mode, to trigger the Name-Change-Rule.</param>
        /// <returns></returns>
        public DataType CreateDataType(string cType, Store store, out string Name)
        {
            if (cache.ContainsKey(cType))
            {
                DataType cached = cache[cType];

                DataType tcopy = new DataType(store);
                tcopy.Name = cType;
                tcopy.CType = cType;
                tcopy.DslType = cached.DslType;

                Name = cached.Name;

                return tcopy;
            }

            DataType t = new DataType(store);
            t.Name = cType;
            t.CType = cType;

            // try to guess the type
            if (new[] { "byte", "int", "uint", "float", "double", "single" }.Any(x => cType.Contains(x)))
                t.DslType = DslDataType.Number;
            else if (new[] { "bool", "boolean" }.Contains(cType))
                t.DslType = DslDataType.Boolean;
            else if (new[] { "string", "String", "char" }.Contains(cType))
                t.DslType = DslDataType.String;
            else
                t.DslType = DslDataType.Other;

            Name = cType;
            return t;
        }
    }
}
