using BenjaminSchroeter.Dsl.UserRestrictions;
using BenjaminSchroeter.DslTools;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    // Most of the elements of this DSL cannot be deleted by a normal user.
    // Here is the IDynamicCanDelete interface implemented to allow or
    // disallow the deletion of the classes from the model by checking
    // the current restriction mode

    // all restrictions in a short form:
    //  - Elements can only be deleted if the user opens the file as a .hardware oder .driver file
    //    - Ports (Input, Output, Event)
    //    - Variables
    //    - Parameter
    //    - Fields
    //  - same for the connectors
    //
    //  - Hardware platforms can only be deleted in a .driver or .driverinfo file (but they should never be there)
    //  - Driver platfomrs can only be deleted in a .hardware or .hardwareinfo file 
    //    - they should never be in a .hardware file
    //    - in a .hardwareinfo file the user can remove the driver with deleting the platform

    #region Shapes
    partial class Port : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int) RestrictionMode.Hardware || 
                   currentMode == (int) RestrictionMode.Driver ||
                   currentMode == (int) RestrictionMode.Original; 
        }
    }

    partial class Variable : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.Original; 
        }
    }

    partial class DataType : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.Original;
        }
    }
    #endregion

    #region compartment entries
    partial class FieldBase : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.Original; 
        }
    }

    partial class EnableableFieldBase : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.Original;
        }
    }
    #endregion

    #region connectors
    partial class EnableableRelationship : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.Original;
        }
    }
    #endregion

    #region platforms
    partial class HardwarePlatform : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.DriverInfo ||
                   currentMode == (int)RestrictionMode.Original;
        }   
    }

    partial class DriverPlatform : IDynamicCanDelete
    {
        public bool CanDelete()
        {
            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            return currentMode == (int)RestrictionMode.Hardware ||
                   currentMode == (int)RestrictionMode.HardwareInfo ||
                   currentMode == (int)RestrictionMode.Original;
        }
    }
    #endregion
}
