using BenjaminSchroeter.Dsl.UserRestrictions;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    // Delegates the CanMerge... functions to the UserRestrictionProvider.
    //
    // In a .hardwareinfo oder .driverinfo file the user cannot and any of the
    // classes that can be found in this file

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "Parameter", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "Parameter", true)]
    partial class Port
    {
        private bool CanMergeParameter(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            return UserRestrictionProvider.AllowMerge(this, "Parameter");
        }
    }

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "InputField", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "InputField", true)]
    partial class Input
    {
        private bool CanMergeInputField(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            return UserRestrictionProvider.AllowMerge(this, "InputField");
        }
    }

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "OutputField", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "OutputField", true)]
    partial class Output 
    {
        private bool CanMergeOutputField(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            return UserRestrictionProvider.AllowMerge(this, "OutputField");
        }
    }

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "EventField", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "EventField", true)]
    partial class Event
    {
        private bool CanMergeEventField(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            return UserRestrictionProvider.AllowMerge(this, "EventField");
        }
    }

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "Variable", true)]
    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "Input", true)]
    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "Event", true)]
    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "Output", true)]

    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "Variable", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "Input", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "Event", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "Output", true)]
    partial class Platform
    {
        private bool CanMergeVariable(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            if (!IsMergeingFromToolbox(prototype))
                return false;

            return UserRestrictionProvider.AllowMerge(this, "Variable");
        }

        private bool CanMergeInput(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            if (!IsMergeingFromToolbox(prototype))
                return false;

            return UserRestrictionProvider.AllowMerge(this, "Input");
        }

        private bool CanMergeEvent(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            if (!IsMergeingFromToolbox(prototype))
                return false;

            return UserRestrictionProvider.AllowMerge(this, "Event");
        }

        private bool CanMergeOutput(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            if (!IsMergeingFromToolbox(prototype))
                return false;
            
            return UserRestrictionProvider.AllowMerge(this, "Output");
        }

        private bool IsMergeingFromToolbox(ElementGroupPrototype prototype)
        {
            return CopyOriginContext.GetCopyOrigin(prototype) == "Toolbox";
        }
    }

    [RestrictElementMerge((int)RestrictionMode.HardwareInfo, "DataType", true)]
    [RestrictElementMerge((int)RestrictionMode.DriverInfo, "DataType", true)]
    [RestrictElementMerge((int)RestrictionMode.Driver, "DataType", false)]
    [RestrictElementMerge((int)RestrictionMode.Hardware, "DataType", false)]
    partial class HardwareDescriptionModel
    {
        private bool CanMergeDataType(ProtoElementBase element, ElementGroupPrototype prototype)
        {
            return UserRestrictionProvider.AllowMerge(this, "DataType");
        }
    }
}
