using System;
using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class EnableableRelationship
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateEnabledFlags(ValidationContext context)
        {
            // get source and target of this relationship
            EnableableDomainClass source = GetSourceAsFieldOrShape();
            EnableableDomainClass target = GetTargetAsFieldOrShape();

            if ( source == null || target == null )
                return;

            // the two classes connected by this relationship must not be both enabled
            if ( source.Enabled && target.Enabled )
            {
                string msg = string.Format("'{0}' and '{1}' must not both be enabled since an Enableable Relationship exists.", source.Name, target.Name);
                context.LogError(msg, "EnableableRelationship.ValidateEnabledFlags", source, target, this);                
            }
        }

        /// <summary>
        /// Gets the EnableableDomainClass that is the source of this relationship.
        /// This can be a Field or a Shape mapped class.
        /// </summary>
        public EnableableDomainClass GetSourceAsFieldOrShape()
        {
            return GetXAsFieldOrShape(this.Source, this.SourceEntry);
        }
        /// <summary>
        /// Gets the EnableableDomainClass that is the target of this relationship.
        /// This can be a Field or a Shape mapped class.
        /// </summary>
        public EnableableDomainClass GetTargetAsFieldOrShape()
        {
            return GetXAsFieldOrShape(this.Target, this.TargetEntry);
        }
        private EnableableDomainClass GetXAsFieldOrShape(EnableableShapeMappedDomainClass element, Guid entry)
        {
            if (entry.Equals(Guid.Empty))
                return element;

            if (element is Input)
                return (element as Input).Fields.FirstOrDefault(x => x.Guid.Equals(entry));

            if (element is Output)
                return (element as Output).Fields.FirstOrDefault(x => x.Guid.Equals(entry));

            return null;
        }
    }
}
