using System.Collections.Generic;
using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class Event
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateFields(ValidationContext context)
        {
            ValidationHelper.CheckNameUniqueness(context,
                                                 this.Fields,
                                                 "Event.ValidateFields",
                                                 x => x.Name);
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateMultiplicity(ValidationContext context)
        {
            if (this.Multiplicity <= 0)
            {
                string msg = string.Format("The Multiplicity of the Event '{0}' must be equal or greater than 1.", this.Name) ;
                context.LogError(msg, "DataType.ValidateMultiplicity", this);
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateFunctionParameterUniqueness(ValidationContext context)
        {
            List<int> callPos = new List<int>();
            foreach (EventField field in this.Fields)
                callPos.Add(field.CallParameter.ParameterPos);

            this.ValidateFunctionParameterUniqueness(context, callPos, new List<int>());
        }
    }
}