using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class FieldBase
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateName(ValidationContext context)
        {
            if (this.Name.Trim() == "")
            {
                string type = this.GetType().Name;
                string msg = string.Format("The name of a {0} must not be empty.", type);
                context.LogError(msg, "FieldBase.ValidateName", this);
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateType(ValidationContext context)
        {
            if (this.Type == null || string.IsNullOrEmpty(this.Type.Value))
            {
                string msg = string.Format("The Type of the {1} '{0}' must not be empty.", this.Name, this.GetType().Name);
                context.LogError(msg, "Variable.ValidateType", this);
            }
            else
            {
                HardwareDescriptionModel model = (HardwareDescriptionModel) this.Partition.ElementDirectory.FindElements(HardwareDescriptionModel.DomainClassId)[0];
                if (model.DataTypes.FirstOrDefault(d => d.Name == this.Type.Value) == null)
                {
                    string msg = string.Format("The Type '{0}' of the {2} '{1}' is not defined in this Hardware Description.", this.Type.Value, this.Name, this.GetType().Name);
                    context.LogError(msg, "Variable.ValidateType", this);
                }
            }
        }
    }
}
