using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Dsl;
using BenjaminSchroeter.Dsl.Graphs;
using BenjaminSchroeter.Dsl.UserRestrictions;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class HardwareDescriptionModel
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidatePlatformCount(ValidationContext context)
        {
            // validate the platforms and count of them according 
            // to the RestrictionMode (which is based on the file extension).

            int currentMode = UserRestrictionProvider.GetRestrictionMode(this.Store);
            bool isDriver = currentMode == (int)RestrictionMode.Driver ||
                   currentMode == (int)RestrictionMode.DriverInfo; 

            if ( isDriver )
            {
                if (this.Platforms.OfType<HardwarePlatform>().Count() > 0)
                {
                    string msg = "The Driver Description must not contain any Hardware Platform.";
                    context.LogError(msg, "HardwareDescriptionModel.ValidatePlatformCount", this);
                }

                if (this.Platforms.OfType<DriverPlatform>().Count() != 1)
                {
                    string msg = "The Driver Description must contain exactly one Driver Platform.";
                    context.LogError(msg, "HardwareDescriptionModel.ValidatePlatformCount", this);
                }
            }
            else
            {
                if (this.Platforms.OfType<HardwarePlatform>().Count() != 1)
                {
                    string msg = "The Hardware Description must contain exactly one Hardware Platform.";
                    context.LogError(msg, "HardwareDescriptionModel.ValidatePlatformCount", this);
                }
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateNames(ValidationContext context)
        {
            ValidationHelper.CheckNameUniqueness(context,
                                                 this.DataTypes,
                                                 "HardwareDescriptionModel.ValidateNames",
                                                 r => r.Name);
        }
    }
}
