using System.Collections.Generic;
using System.Linq;
using BenjaminSchroeter.Dsl;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class Input
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateFields(ValidationContext context)
        {
            ValidationHelper.CheckNameUniqueness(context,
                            this.Fields,
                            "Input.ValidateFields",
                            x => x.Name);

            if ( this.Fields.Count == 0)
            {
                string msg = string.Format("The Input '{0}' must contain at least on filed.", this.Name);
                context.LogError(msg, "Input.ValidateFields", this);                
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateFunctionParameterUniqueness(ValidationContext context)
        {
            List<int> callPos = new List<int>();
            List<int> initPos = new List<int>();

            foreach (InputField field in this.Fields)
            {
                CallParameter c = field.CallParameter;
                callPos.Add(c.ParameterPos);
                initPos.Add(c.EnabledParameterPos);
            }

            this.ValidateFunctionParameterUniqueness(context,callPos,initPos);
        }
        
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCall(ValidationContext context)
        {
            CallParameter c = this.ValidateInputOutputCall(context, this.CallParameter);

            if (!c.HasFunction)
            {
                bool eachFieldHasCall = this.Fields.All(x => (x.CallParameter.HasFunction));
                if (!eachFieldHasCall)
                {
                    string msg = string.Format("The Call Property of {1} '{0}' is invalid. You must provide a Function Name for the Shape or one for every single Field.", this.Name, this.GetType().Name);
                    context.LogError(msg, "Input.ValidateCall", this);
                }
            }
        }
    }
}