using System;
using BenjaminSchroeter.Flow.HardwareDescription.Data;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class Parameter
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCall(ValidationContext context)
        {
            CallParameter c = this.CallParameter;
            
            if (c.HasFunction || c.HasEnabledParameter)
            {
                string msg = string.Format("The Call Property of Parameter '{0}' of '{1}' is invalid. You should only provide the position in the parameter list.", this.Name, this.Port.Name);
                context.LogError(msg, "Parameter.ValidateCall", this);
            }

            if(!c.HasParameter)
            {
                string msg = string.Format("The Call Property of Parameter '{0}' of '{1}' does not contain the position in the parameter list.", this.Name, this.Port.Name);
                context.LogWarning(msg, "Parameter.ValidateCall", this);
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateValue(ValidationContext context)
        {
            DslDataType dataType = this.Type.AsDslType(this.Store);
            switch (dataType)
            {
                case DslDataType.Boolean:
                    bool x;
                    if (!Boolean.TryParse(this.Value, out x))
                    {
                        string msg = string.Format("The Value ('{1}') of Parameter '{0}' is not a boolean value. Please use 'True' or 'False'.", this.Name, this.Value);
                        context.LogError(msg, "Parameter.ValidateValue", this);
                    }
                    break;

                case DslDataType.Number:
                    int i;
                    if ( !Int32.TryParse(this.Value, out i))
                    {
                        string msg = string.Format("The Value ('{1}') of Parameter '{0}' is not a numeric value.", this.Name, this.Value);
                        context.LogError(msg, "Parameter.ValidateValue", this);
                    }                    
                    break;
                
                case DslDataType.String:
                    // every string can be parsed to string ;)
                    break;

                case DslDataType.Record:
                case DslDataType.Other:
                    string msg2 = string.Format("For the Parameter '{1}' of a {0} you must not use a DataType that maps to a Record.", this.Port.GetType().Name, this.Name);
                    context.LogError(msg2, "Parameter.ValidateValue", this);
                    break;
                
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
