using System.Linq;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ValidationState(ValidationState.Enabled)]
    partial class Variable
    {
        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateType(ValidationContext context)
        {
            if (this.Type == null || string.IsNullOrEmpty(this.Type.Value))
            {
                string msg = string.Format("The Type of the Variable '{0}' must not be empty.", this.Name);
                context.LogError(msg, "Variable.ValidateType", this);
            }
            else
            {
                if(this.Platform.HardwareDescriptionModel.DataTypes.FirstOrDefault( d => d.Name == this.Type.Value ) == null)
                {
                    string msg = string.Format("The Type '{0}' of the Variable '{1}' is not defined in this Hardware Description.", this.Type.Value, this.Name);
                    context.LogError(msg, "Variable.ValidateType", this);                    
                }
            }
        }

        [ValidationMethod(ValidationCategories.Menu | ValidationCategories.Save | ValidationCategories.Open)]
        private void ValidateCalls(ValidationContext context)
        {
            bool get = this.CallGetter != "";
            bool set = this.CallSetter != "";

            if ( get ) ValidateCall(context, this.CallParameterGetter, "Get");
            if ( set ) ValidateCall(context, this.CallParameterSetter, "Set");

            if (!get && !set)
            {
                string msg = string.Format("The Call Properties of Variable '{0}' are both empty. Please provide at least a Getter or a Setter (or both).", this.Name);
                context.LogError(msg, "Variable.ValidateCalls", this);                                                    
            }
        }
        private void ValidateCall(ValidationContext context, CallParameter call, string name)
        {
            if ( call.HasEnabledParameter || call.HasParameter || !call.HasFunction)
            {
                string msg = string.Format("The {1} Call Property of Variable '{0}' is invalid. Please provide only the Function Name.", this.Name, name);
                context.LogError(msg, "Variable.ValidateCall", this);                                    
            }
        }
    }
}
