using System.ComponentModel.Design;
using System.Diagnostics;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    /// <summary>
    /// Handler methods for the Show and Hide Connectors command.
    /// </summary>
    class ShowHideConnectorsCommandHandler
    {
        public const uint cmdId = 0x2000;

        /// <summary>
        /// Queries the status for the ImportDriver command.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="isHardwaredescription">if set to <c>true</c> a hardwaredescription model is selected.</param>
        /// <param name="isSwimlaneSelected">if set to <c>true</c> a is swimlane selected.</param>
        public static void QueryStatus(object sender, bool isHardwaredescription, bool isSwimlaneSelected)
        {
            MenuCommand cmd = sender as MenuCommand;
            Debug.Assert(cmd != null, "sender is not a MenuCommand");

            cmd.Enabled = cmd.Visible = isHardwaredescription && isSwimlaneSelected;
        }

        /// <summary>
        /// Invokes the command.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="swimlane">A swimlane of the HardwareDescription model.</param>
        public static void Invoke(object sender, SwimlaneShape swimlane)
        {
            MenuCommand cmd = sender as MenuCommand;
            Debug.Assert(cmd != null, "sender is not a MenuCommand");

            cmd.Checked = !cmd.Checked;

            // show or hide every connector
            foreach (EnableableRelationshipConnector element in swimlane.Partition.ElementDirectory.FindElements(EnableableRelationshipConnector.DomainClassId))
            {
                if ( cmd.Checked )
                    element.Show();
                else
                    element.Hide();

            }
            
            // invalidate the model to repaint the connectors
            swimlane.Diagram.Invalidate(true);
            swimlane.Diagram.Show();
        }
    }
}
