using System.ComponentModel;
using System.Drawing.Design;
using System.IO;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    [ComVisible(true)]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    [Guid("2F030CDD-3AD0-424E-BD54-F9A1E4DE9D6C")]
    public class FirmwareParserOptionPage : DialogPage
    {
        private string doxygenExecutable;

        /// <summary>
        /// Gets or sets the full path to the doxygen executable.
        /// </summary>
        /// <value>The doxygen executable.</value>
        [Description("The full path of the Doxygen executable (doxygen.exe).")]
        [Editor(typeof(ExeFileNameEditor), typeof(UITypeEditor))]
        public string DoxygenExecutable
        {
            get
            {
                if ( string.IsNullOrEmpty(doxygenExecutable) || !File.Exists(doxygenExecutable) )
                {
                    // no path to the doxygen executalbe is given. 
                    // try to find one by ourself

                    try
                    {
                        string path = Path.GetDirectoryName(this.GetType().Assembly.Location);
                        DirectoryInfo di = new DirectoryInfo(path);
                        if ( di.Exists )
                        {
                            FileInfo[] files = di.GetFiles("doxygen.exe", SearchOption.AllDirectories);
                            if (files.Length >= 1)
                                doxygenExecutable = files[0].FullName;
                        }
                    }
                    catch {}
                }

                return doxygenExecutable;
            }
            set
            {
                doxygenExecutable = value;
            }
        }

        //TIP 1: If you want to get access this option page from a VS Package use this snippet on the VsPackage class:
        //FirmwareParserOptionPage optionPage = this.GetDialogPage(typeof(FirmwareParserOptionPage)) as FirmwareParserOptionPage;

        //TIP 2: If you want to get access this option page from VS Automation copy this snippet:
        //DTE dte = GetService(typeof(DTE)) as DTE;
        //EnvDTE.Properties props = dte.get_Properties("HardwareDescription", "FirmwareParser");		
    }
}

