using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using BenjaminSchroeter.Dsl.CompartmentMapping;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Diagrams;
using Microsoft.VisualStudio.Modeling.Shell;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    partial class HardwareDescriptionCommandSet
    {
        protected override IList<MenuCommand> GetMenuCommands()
        {
            IList<MenuCommand> commands = base.GetMenuCommands();

            // remove some commands
            CompartmentMappingUtil.RemoveRerouteCommand(commands);
            CompartmentMappingUtil.RemoveCompartmentShapeExpandCollapseCommand(commands);

            // and add some
            commands.Add(CreateImportDriverCommand());
            commands.Add(CreateShowHideConnectorsCommand());
            commands.Add(CreateShowExplorerCommand());

            return commands;
        }

        #region create commands
        /// <summary>
        /// Creates the "import driver..." command.
        /// </summary>
        /// <remarks>
        /// Visible on the context menu in the solution explorer on .hardware and .driver files
        /// </remarks>
        /// <returns></returns>
        private DynamicStatusMenuCommand CreateImportDriverCommand()
        {
            CommandID menuCommandID = new CommandID(CommandConst.guidHardwareDescription, (int)ImportDriverCommandHandler.cmdId);            

            DynamicStatusMenuCommand menuItem = new DynamicStatusMenuCommand(
                delegate(object sender, EventArgs e) { ImportDriverCommandHandler.QueryStatus(sender, this.CurrentDocData is HardwareDescriptionDocData, GetSelectedSwimlane() != null, this.CurrentDocData ==null?"":this.CurrentDocData.FileName, this.ServiceProvider); },
                delegate { ImportDriverCommandHandler.Invoke(GetSelectedSwimlane(), this.ServiceProvider); },
                menuCommandID);

            return menuItem;
        }

        /// <summary>
        /// Creates the "show / hide connectors" command.
        /// </summary>
        /// <remarks>
        /// On the context menu of Hardware description model panes.
        /// </remarks>
        /// <returns></returns>
        private DynamicStatusMenuCommand CreateShowHideConnectorsCommand()
        {
            CommandID menuCommandID = new CommandID(CommandConst.guidHardwareDescription, (int)ShowHideConnectorsCommandHandler.cmdId );

            DynamicStatusMenuCommand menuItem = new DynamicStatusMenuCommand(
                delegate(object sender, EventArgs e) { ShowHideConnectorsCommandHandler.QueryStatus(sender, this.CurrentDocData is HardwareDescriptionDocData, GetSelectedSwimlane() != null); },
                delegate(object sender, EventArgs e) { ShowHideConnectorsCommandHandler.Invoke(sender, GetSelectedSwimlane()); },
                menuCommandID);

            menuItem.Checked = true;
            return menuItem;
        }

        /// <summary>
        /// Creates the "show hardwaredescription explorer" command.
        /// </summary>
        /// <remarks>
        /// On the context menu of Hardware description model pane.
        /// </remarks>        
        /// /// <returns></returns>
        private MenuCommand CreateShowExplorerCommand()
        {
            CommandID menuCommandID = new CommandID(CommandConst.guidHardwareDescription, (int)CommandConst.cmdIdShowExplorer);

            DynamicStatusMenuCommand menuItem = new DynamicStatusMenuCommand(
                 delegate(object sender, EventArgs e) { ShowHideConnectorsCommandHandler.QueryStatus(sender, this.CurrentDocData is HardwareDescriptionDocData, GetSelectedSwimlane() != null); },
                OnMenuViewModelExplorer,
                menuCommandID); 
            
            return menuItem;
        }

        /// <summary>
        /// Gets the selected swimlane from the CurrentDocumentSelection
        /// </summary>
        /// <remarks>Used by some of the command handlers above</remarks>
        /// <returns></returns>
        private SwimlaneShape GetSelectedSwimlane()
        {
            SwimlaneShape swimlane = null;

            foreach (ModelElement element in this.CurrentDocumentSelection)
                if (element is SwimlaneShape)
                    swimlane = element as SwimlaneShape;
                else
                    return null;

            return swimlane;
        }
        #endregion
    }
}
