using System;
using System.ComponentModel.Design;
using BenjaminSchroeter.Dsl.UserRestrictions;
using BenjaminSchroeter.Flow.HardwareDescription.DataService;
using Microsoft.VisualStudio.Modeling.Shell;
using Microsoft.VisualStudio.Shell;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    // Option page
    [ProvideOptionPage(typeof(FirmwareParserOptionPage), "Flow", "FirmwareParser", 21543, 13216, true)]

    // Editor for .hardwareinfo
    [ProvideEditorExtension(typeof(HardwareDescriptionEditorFactory), "." + Constants.DesignerFileExtensionHardwareInfo, 32)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionHardwareInfo, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.CSharpProjectGuid, FileOptions = RelatedFileType.FileName)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionHardwareInfo, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.VisualBasicProjectGuid, FileOptions = RelatedFileType.FileName)]

    // Editor for .dirver
    [ProvideEditorExtension(typeof(HardwareDescriptionEditorFactory), "." + Constants.DesignerFileExtensionDriverDescription, 32)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionDriverDescription, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.CSharpProjectGuid, FileOptions = RelatedFileType.FileName)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionDriverDescription, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.VisualBasicProjectGuid, FileOptions = RelatedFileType.FileName)]

    // Editor for .driverinfo
    [ProvideEditorExtension(typeof(HardwareDescriptionEditorFactory), "." + Constants.DesignerFileExtensionDriverInfo, 32)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionDriverInfo, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.CSharpProjectGuid, FileOptions = RelatedFileType.FileName)]
    [ProvideRelatedFile("." + Constants.DesignerFileExtensionDriverInfo, Constants.DefaultDiagramExtension, ProjectSystem = ProvideRelatedFileAttribute.VisualBasicProjectGuid, FileOptions = RelatedFileType.FileName)]

    // services
    [ProvideService(typeof(SHardwareDescriptionDataService))]
    [ProvideService(typeof(SFirmwareParser))]

    // package load key
    [ProvideLoadKey("Standard", "1.0", "Flow Hardware Description", "Benjamin Schroeter", 200)]
    partial class HardwareDescriptionPackage
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HardwareDescriptionPackage"/> class.
        /// </summary>
        public HardwareDescriptionPackage() : base()
        {
            IServiceContainer serviceContainer = this;

            // register services
            serviceContainer.AddService(typeof(SHardwareDescriptionDataService), createDataService, true);
            serviceContainer.AddService(typeof (SFirmwareParser), createFirmwareParserService, true);
        }

        /// <summary>
        /// Initialization method called by the package base class when this package is loaded.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            // create ErrorListProvider
            errorList = new ErrorListProvider(this)
            {
                ProviderName = "Hardware Description",
                ProviderGuid = new Guid("9EC14BE5-D6E1-4b38-BB12-57A0AA6712C8")
            };

            // user restrictions
            UserRestrictionProvider.RegisterAllRestrictedElements<HardwareDescriptionDiagram>();
        }

        #region services
        /// <summary>
        /// Creates the SHardwareDescriptionDataService.
        /// </summary>
        /// <param name="container">The container.</param>
        /// <param name="serviceType">Type of the service.</param>
        /// <returns></returns>
        private object createDataService(IServiceContainer container, Type serviceType)
        {
            if (container != this)
                return null;

            if (serviceType == typeof(SHardwareDescriptionDataService))
                return new HardwareDescriptionDataService(this);

            return null;
        }

        /// <summary>
        /// Creates the SFirmwareParser service.
        /// </summary>
        /// <param name="container">The container.</param>
        /// <param name="serviceType">Type of the service.</param>
        /// <returns></returns>
        private object createFirmwareParserService(IServiceContainer container, Type serviceType)
        {
            if (container != this)
                return null;

            if (serviceType == typeof(SFirmwareParser))
                return new FirmwareParserService(this, this.GetDialogPage(typeof (FirmwareParserOptionPage)) as FirmwareParserOptionPage, errorList);

            return null;
        }
        #endregion

        /// <summary>
        /// Provider to access the error list for this package
        /// </summary>
        private ErrorListProvider errorList;
    }
}