using System.Diagnostics;
using System.Drawing.Design;
using BenjaminSchroeter.Dsl.UserRestrictions;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Modeling.Design;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell.Interop;

namespace BenjaminSchroeter.Flow.HardwareDescription
{
    partial class HardwareDescriptionDocView : IVsToolboxUser
    {
        /// <summary>
        /// Determines whether the specified ToolboxItem is supported with the current Restriction Mode.
        /// </summary>
        /// <param name="pDO">The ModelingToolboxItem warapped in a IDataObject</param>
        /// <returns></returns>
        public new int IsSupported(IDataObject pDO)
        {
            bool supported = base.IsSupported(pDO) == VSConstants.S_OK;

            if (supported)
            {
                IToolboxService toolboxService = base.GetService(typeof(IToolboxService)) as IToolboxService;
                Debug.Assert(toolboxService != null, "Cannot receive the IToolboxService service.");

                // get the toolbox item
                ModelingToolboxItem item = toolboxService.DeserializeToolboxItem(pDO) as ModelingToolboxItem;

                if (item != null && this.Diagram != null)
                {
                    // check the attributes of this diagram
                    foreach (RestrictToolboxItemUsingAttribute attribute in this.Diagram.GetType().GetCustomAttributes(typeof(RestrictToolboxItemUsingAttribute), true))
                    {
                        if (UserRestrictionProvider.GetRestrictionMode(this.Diagram.Store) == attribute.RestrictionMode
                            && item.Id == attribute.ItemId
                            && item.TabNameId == attribute.TabNameId)
                        {
                            supported = false;
                            break;
                        }
                    }
                }
            }

            // return OK for supported, otherwise FALSE 
            return supported ? VSConstants.S_OK : VSConstants.S_FALSE;
        }
    }
}
