﻿using System.Collections.Generic;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Dsl
{
    /// <summary>
    /// Some utilites for the DSL Decorators
    /// </summary>
    public class DecoratorUtils
    {
        /// <summary>
        /// Makes a decorator displaying multi line text.
        /// </summary>
        /// <remarks>Call this method from the InitializeDecorators method of da Shape class.</remarks>
        /// <example>
        /// protected override void InitializeDecorators(IList<ShapeField> shapeFields, IList<Decorator> decorators)
        /// {
        ///    base.InitializeDecorators(shapeFields, decorators);
        ///
        ///    // enable multi line text on the comment
        ///    DecoratorUtils.MakeDecoratorMultiLine(shapeFields, "TextDecorator");
        /// }
        /// </example>
        /// <param name="decoratorName">Name of the decorator.</param>
        public static void MakeDecoratorMultiLine(IList<ShapeField> shapeFields, string decoratorName)
        {
            // for details refere to the book "Domain-Specific Development with Visual Studio DSL Tools" by Cook et.al. page 173
            // or the Document "Customizing features in Microsoft Tools for Domain Specific Languages" that
            // comes with your installation of the VSSDK.

            TextField field = ShapeElement.FindShapeField(shapeFields, decoratorName) as TextField;
            if (field == null)
                return;

            field.DefaultAutoSize = false;
            field.DefaultMultipleLine = true;

            field.AnchoringBehavior.Clear();
            field.AnchoringBehavior.SetLeftAnchor(AnchoringBehavior.Edge.Left, 0.01);
            field.AnchoringBehavior.SetRightAnchor(AnchoringBehavior.Edge.Right, 0.01);
            field.AnchoringBehavior.SetBottomAnchor(AnchoringBehavior.Edge.Bottom, 0.01);
            field.AnchoringBehavior.SetTopAnchor(AnchoringBehavior.Edge.Top, 0.01);
        }
    }
}
