﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Validation;

namespace BenjaminSchroeter.Dsl
{
    public class ValidationHelper
    {
        public static void CheckNameUniqueness<TYPE>(ValidationContext context,
                                                     IEnumerable<TYPE> uniqueElements,
                                                     string errorCode,
                                                     Func<TYPE, string> getName) where TYPE : ModelElement
        {
            CheckUniqueness(context,uniqueElements,"Name",errorCode,getName, getName);
        }
        
        public static void CheckUniqueness<KEY, TYPE>(ValidationContext context,
                                                      IEnumerable<TYPE> uniqueElements,
                                                      string keyName,
                                                      string errorCode,
                                                      Func<TYPE, KEY> getKey,
                                                      Func<TYPE, string> getName) where TYPE : ModelElement
        {
            Dictionary<KEY, List<TYPE>> cache = new Dictionary<KEY, List<TYPE>>();

            foreach (TYPE element in uniqueElements)
            {
                KEY key = getKey(element);

                if (!cache.ContainsKey(key))
                    cache.Add(key, new List<TYPE>());

                cache[key].Add(element);
            }

            foreach (KeyValuePair<KEY, List<TYPE>> pair in cache.Where(x => x.Value.Count > 1))
            {
                List<string> names = new List<string>();
                foreach (TYPE element in pair.Value)
                {
                    string n = getName(element);
                    if (!names.Contains(n))
                        names.Add(n);
                }

                names.Sort();
                string allNames = string.Join(" ,", names.ToArray());

                string msg = string.Format("The {0} ({1}) of the following elements must be unique: {2}", keyName, pair.Key, allNames);
                context.LogError(msg, errorCode, pair.Value.ToArray());
            }
        }
    }
}