﻿using System;
using System.Collections.Generic;
using EnvDTE;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;

namespace BenjaminSchroeter.VsxTools
{
    /// <summary>
    /// Some extension methods for the IVsHierarchy interface
    /// </summary>
    public static class IVsHierarchyExtensions
    {
        /// <summary>
        /// Gets the root hierarchy item of this hierarchy
        /// </summary>
        /// <param name="hierarchy">The IVsHierarchy instance.</param>
        /// <returns></returns>
        public static HierarchyItem GetRoot(this IVsHierarchy hierarchy)
        {
            return new HierarchyItem(hierarchy, VSConstants.VSITEMID_ROOT);
        }

        /// <summary>
        /// Finds all items in this hierarchy that represent files on disk.
        /// </summary>
        /// <param name="hierarchy">The hierarchy.</param>
        /// <returns></returns>
        public static IEnumerable<HierarchyItem> FindFileItems(this IVsHierarchy hierarchy)
        {
            return FindFileItems(hierarchy, x => true);
        }

        /// <summary>
        /// Finds all items in this hierarchy that represent files on disk and fullfill the fileNameFilter predicate
        /// </summary>
        /// <param name="hierarchy">The hierarchy.</param>
        /// <param name="fileNameFilter">The file name filter.</param>
        /// <returns></returns>
        public static IEnumerable<HierarchyItem> FindFileItems(this IVsHierarchy hierarchy, Predicate<string> fileNameFilter)
        {
            foreach (HierarchyItem item in hierarchy.GetRoot().GetChildrenRecursiv())
            {
                string name = item.FullFileName;
                if (name != "" && fileNameFilter(name))
                    yield return item;
            }
        }

        /// <summary>
        /// If the IVsHierarchy represents a project, this project
        /// will be returend a a EnvDTE.Project.
        /// </summary>
        /// <param name="hierarchy">The hierarchy.</param>
        /// <returns></returns>
        public static Project ToDteProject(this IVsHierarchy hierarchy)
        {
            object prjObject = null;
            hierarchy.GetProperty(VSConstants.VSITEMID_ROOT, (int)__VSHPROPID.VSHPROPID_ExtObject, out prjObject);
            return prjObject as Project;
        }
    }
}
