﻿using System.Collections.Generic;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell.Interop;

namespace BenjaminSchroeter.VsxTools
{
    /// <summary>
    /// Some extension methods for the IVsToolbox2Extensions interface
    /// </summary>
    public static class IVsToolbox2Extensions
    {
        /// <summary>
        /// Gets all tabs of the toolbox
        /// </summary>
        /// <param name="toolbox">The IVsToolbox2 instance.</param>
        /// <returns></returns>
        public static IEnumerable<string> GetTabs(this IVsToolbox2 toolbox)
        {
            IEnumToolboxTabs pEnum;
            toolbox.EnumTabs(out pEnum);

            uint pceltFetched = 1;
            string[] rgelt = new string[1];

            while (pceltFetched > 0)
            {
                pEnum.Next(1, rgelt, out pceltFetched);

                if (pceltFetched > 0)
                    yield return rgelt[0];
            }
        }

        /// <summary>
        /// Gets the toolbox items of a toolbox tab.
        /// </summary>
        /// <param name="toolbox">The IVsToolbox2 instance.</param>
        /// <param name="tab">The name of the toolbox tab.</param>
        /// <returns></returns>
        public static IEnumerable<IDataObject> GetItems(this IVsToolbox2 toolbox, string tab)
        {
            IEnumToolboxItems pEnum;
            toolbox.EnumItems(tab, out pEnum);

            uint pceltFetched = 1;
            IDataObject[] rgelt = new IDataObject[1];

            while (pceltFetched > 0)
            {
                pEnum.Next(1, rgelt, out pceltFetched);

                if (pceltFetched > 0)
                    yield return rgelt[0];
            }
        }
    }
}
