﻿using System;
using System.Collections.Generic;
using System.Text;

namespace BenjaminSchroeter
{
    /// <summary>
    /// A very simple class to do some (lame) performance measurements.
    /// </summary>
    /// <example>
    /// Stopwatch s = new Stopwatch("some name");
    /// /* do some time consuming work */
    /// s.NewLap();
    /// /* do some more time consuming work */
    /// s.NewLap();
    /// Console.WriteLine(s.PrintTimes());
    /// </example>
    public class Stopwatch
    {
        private List<DateTime> laps = new List<DateTime>();
        public string Name { get; private set;}

        /// <summary>
        /// Initializes a new instance of the <see cref="Stopwatch"/> class.
        /// </summary>
        /// <param name="name">The name of the stopwatch.</param>
        public Stopwatch(string name)
        {
            this.Name = name;
            Restart();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Stopwatch"/> class with empty name.
        /// </summary>
        public Stopwatch() : this("")
        {
        }

        /// <summary>
        /// Restarts this stopwatch.
        /// </summary>
        private void Restart()
        {
            laps = new List<DateTime>();
            NewLap();
        }

        /// <summary>
        /// Adds a new lap.
        /// </summary>
        public void NewLap()
        {
            laps.Add(DateTime.Now);
        }

        /// <summary>
        /// Prints out the recorded laps.
        /// </summary>
        /// <returns></returns>
        public string PrintTimes()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("------------ " + Name + " ----------------");
            for (int i = 1; i < laps.Count; i++)
            {
                TimeSpan t = laps[i] - laps[i - 1];
                sb.AppendLine(string.Format("{0}.) {1} ms", i, t.TotalMilliseconds));
            }
            return sb.ToString();
        }
    }
}
