using System;
using System.Windows.Forms;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    /// <summary>
    /// Dialog that shows/asks all information needed to create a Flow Platform Setup
    /// </summary>
    public partial class FlowPlatformSetupDialog : Form
    {
        private FlowHardwarePlatformSetup settings;
        private readonly string defaultPath;

        public FlowPlatformSetupDialog()
        {
            InitializeComponent();
            this.settings = new FlowHardwarePlatformSetup();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FlowPlatformSetupDialog"/> class.
        /// </summary>
        /// <param name="settings">The settings used as a default value.</param>
        /// <param name="defaultPath">The default path for the save setup file dialog.</param>
        public FlowPlatformSetupDialog(FlowHardwarePlatformSetup settings, string defaultPath) : this()
        {
            this.settings = settings;
            this.defaultPath = defaultPath;

            this.txtDesc.Text = settings.Description;
            this.txtFile.Text = settings.SetupName;
            this.txtFullName.Text = settings.FullName;
            this.txtShortName.Text = settings.ShortName;
            this.txtUrl.Text = settings.Url;
            this.txtVendorName.Text = settings.VendorName;
            this.txtIcon.Text = settings.Icon;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            settings.Description = this.txtDesc.Text;
            settings.SetupName = this.txtFile.Text;
            settings.FullName = this.txtFullName.Text;
            settings.ShortName = this.txtShortName.Text;
            settings.Url = this.txtUrl.Text;
            settings.VendorName = this.txtVendorName.Text;
            settings.Icon = this.txtIcon.Text;

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }

        private void btnSelectFile_Click(object sender, EventArgs e)
        {
            SaveFileDialog d = new SaveFileDialog();
            d.InitialDirectory = defaultPath;
            d.Filter = "*.exe|*.exe";
            
            if ( d.ShowDialog(this) != System.Windows.Forms.DialogResult.OK )
                return;

            this.txtFile.Text = d.FileName;
        }

        private void btnSelectIcon_Click(object sender, EventArgs e)
        {
            OpenFileDialog d = new OpenFileDialog();
            d.InitialDirectory = defaultPath;
            d.Filter = "*.ico|*.ico";

            if (d.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
                return;

            this.txtIcon.Text = d.FileName;
        }
    }
}
