using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using EnvDTE;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    /// <summary>
    /// Dialog that shows all files of a Dte.Project in a treeview with
    /// checkboxes to allow the user to select some of these files
    /// </summary>
    public partial class SelectProjectFilesDialog : Form
    {
        // a flat list of all nodes added to the tree
        private List<TreeNode> allNodes;

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectProjectFilesDialog"/> class.
        /// </summary>
        public SelectProjectFilesDialog()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectProjectFilesDialog"/> class
        /// and fills the treeview with the files of the given project
        /// </summary>
        /// <param name="project">The project.</param>
        /// <param name="hiddenFileExtensions">A list of file extensions that will not be shown in the dialog</param>
        public SelectProjectFilesDialog(Project project, string[] hiddenFileExtensions) : this()
        {
            allNodes = new List<TreeNode>();
            TreeNode node = new TreeNode(project.Name);
            CreateChildren(project.ProjectItems, hiddenFileExtensions, node);
            this.treeView1.Nodes.Add(node);
            allNodes.Add(node);
            node.Expand();
        }

        /// <summary>
        /// Creates the children of the ProjectItem as children of the given tree node
        /// </summary>
        /// <param name="items">The items.</param>
        /// <param name="rootnode">The rootnode.</param>
        private void CreateChildren(ProjectItems items, string[] hiddenFileExtensions, TreeNode rootnode)
        {
            foreach (ProjectItem item in items)
            {
                TreeNode node = new TreeNode(item.Name);

                string fileName = item.get_FileNames(0);
                if ( File.Exists(fileName) )
                    node.Tag = item;

                
                CreateChildren(item.ProjectItems, hiddenFileExtensions, node);
                
                if (!hiddenFileExtensions.Any( s => fileName.EndsWith(s,StringComparison.InvariantCultureIgnoreCase) ) )
                {
                    rootnode.Nodes.Add(node);
                    allNodes.Add(node);
                }
            }
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }

        /// <summary>
        /// Event Method to avoid checking of nodes that do not represent
        /// a valid file on disk (e.g. virtual folders)
        /// </summary>
        private void treeView1_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Tag == null && e.Node.Checked )
                e.Node.Checked = false;
        }

        public IEnumerable<ProjectItem> SelectedProjectItems
        {
            get
            {
                return from n in allNodes
                       where n.Checked && n.Tag != null
                        select (ProjectItem)n.Tag;
            }
        }
    }
}
