using System;
using System.ComponentModel.Design;
using System.Diagnostics;
using BenjaminSchroeter.Flow.HardwareDescription;
using EnvDTE;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    /// <summary>
    /// Comamnd Handler for the "Parse Firmware..." context menu entry
    /// on .hardware and .driver files.
    /// </summary>
    internal class ParseFirmwareHandler
    {
        public const int cmdidParseFirmware = 0x6E1F;
        private const string guidParseFirmwareString = "E7815E18-5837-4138-9935-61C406E04257";
        public static readonly Guid guidParseFirmware = new Guid(guidParseFirmwareString);

        /// <summary>
        /// Queries the status of the command.
        /// </summary>
        /// <remarks>The command will only be visible when exactly one file is selected and this file is a .hardware oder .driver file.</remarks>
        /// <param name="sender">The sender.</param>
        /// <param name="serviceProvider">The service provider.</param>
        public static void QueryStatus(object sender, IServiceProvider serviceProvider)
        {
            MenuCommand cmd = sender as MenuCommand;
            Debug.Assert(cmd != null, "sender is not a MenuCommand");

            DTE dte = serviceProvider.GetService(typeof(_DTE)) as DTE;
            Debug.Assert(dte != null, "DTE");

            // multiple selection or whatever..
            if (dte.SelectedItems.Count != 1)
            {
                cmd.Visible = false;
                return;
            }

            string filename = dte.SelectedItems.Item(1).Name;
            cmd.Visible = filename.EndsWith( Const.FileExtensionHardwareDescription , StringComparison.InvariantCultureIgnoreCase)
                           || filename.EndsWith(Const.FileExtensionDriverDescription, StringComparison.InvariantCultureIgnoreCase); 
        }

        /// <summary>
        /// Invokes the command.
        /// </summary>
        /// <param name="serviceProvider">The service provider.</param>
        public static void Invoke(IServiceProvider serviceProvider)
        {
            DTE dte = serviceProvider.GetService(typeof(_DTE)) as DTE;
            Debug.Assert(dte != null, "DTE");

            IFirmwareParser parser = serviceProvider.GetService(typeof(SFirmwareParser)) as IFirmwareParser;
            Debug.Assert(parser != null, "IFirmwareParser");

            // multiple selection or whatever..
            if (dte.SelectedItems.Count != 1)
                return;

            ProjectItem projectItem = dte.SelectedItems.Item(1).ProjectItem;

            // use the firmware parser service
            parser.ParseFirmwareCommandInvoke(projectItem);
        }
    }
}
