using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using BenjaminSchroeter.VsxTools;
using EnvDTE;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    /// <summary>
    /// Event Handler for the Remove Project Item Event
    /// </summary>
    /// <remarks>Will be registerd from the package.</remarks>
    internal class RemoveProjectItemHandler
    {
        /// <summary>
        /// All File extensions this handler will observe
        /// </summary>
        private static readonly string[] HandelFileTypes = new[] { Const.FileExtensionDataFlow, 
                                                                    Const.FileExtensionDatastructures, 
                                                                    Const.FileExtensionDriverInfo, 
                                                                    Const.FileExtensionDriverDescription, 
                                                                    Const.FileExtensionHardwareInfo, 
                                                                    Const.FileExtensionHardwareDescription };
        /// <summary>
        /// If one of the HandelFileTypes files is removed from 
        /// the Project the following files will be deleted, too
        /// </summary>
        private static readonly string[] DeleteAlsoFileTypes = new[] { "{0}.c", 
                                                                        "{0}.tt", 
                                                                        "{0}.{1}" + Const.FileExtensionDiagramAddition }; // {0} = filename , {1} = extension of removed file

        /// <summary>
        /// The event mehtod. Will be called when the Project Item is removed
        /// </summary>
        /// <param name="item">The item.</param>
        public static void ProjectItemsEvents_ItemRemoved(ProjectItem item)
        {
            // only for our project type
            Project project = item.ContainingProject;
            if (!project.IsScatterWebDslProject())
                return;

            // only for some file types (HandelFileTypes)
            string removedName = item.get_FileNames(0);
            string removedNameWithoutExtension = Path.GetFileNameWithoutExtension(removedName);
            string removedNameExtension = Path.GetExtension(removedName).Trim('.');
            if (!HandelFileTypes.Any(n => removedName.EndsWith(n, StringComparison.InvariantCultureIgnoreCase)))
                return;

            // create patterns to delete
            var pattersToDelete = DeleteAlsoFileTypes.Select(x => string.Format(x, removedNameWithoutExtension, removedNameExtension));

            // find all types to rename
            List<ProjectItem> allProjectItems = project.FindAllFiles();
            
            List<ProjectItem> toRemove = new List<ProjectItem>();
            foreach (ProjectItem i in allProjectItems)
            {
                string filename = i.get_FileNames(0);
                if ( pattersToDelete.Any( x => filename.EndsWith(x,StringComparison.InvariantCultureIgnoreCase)))
                    toRemove.Add(i);
            }
                
            // do the actual removing
            foreach (ProjectItem projectItem in toRemove)
                projectItem.Delete();
        }
    }
}
