using System;
using System.Diagnostics;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    /// <summary>
    /// Comamnd Handler for the "Validate All" button
    /// in the toolbar of the solution explorer
    /// </summary>
    public class ValidateProjectHandler
    {
        public const int cmdidValidateProject = 0x4D72;
        public const string guidValidateProjectString = "975F4CF9-049C-42A0-B8D9-8F643652FD52";
        public static readonly Guid guidValidateProject = new Guid(guidValidateProjectString);

        /// <summary>
        /// Queries the status of the command
        /// <remarks>This command is only visible when a Scatter Web Dsl Project is selected in the solution explorer.</remarks>
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="serviceProvider">The service provider.</param>
        public static void QueryStatus(object sender, IServiceProvider serviceProvider)
        {
            // this command can reuse the query status from the TransformTemplatesHandler
            TransformTemplatesHandler.QueryStatus(sender,serviceProvider);
        }

        /// <summary>
        /// Invokes the command.
        /// </summary>
        /// <param name="serviceProvider">The service provider.</param>
        public static void Invoke(IServiceProvider serviceProvider)
        {
            IGlobalValidation v = serviceProvider.GetService(typeof (SGlobalValidation)) as IGlobalValidation;
            Debug.Assert(v != null, "IGlobalValidation");
            v.Validate();
        }
    }
}