using System;
using System.Linq;
using BenjaminSchroeter.VsxTools;
using EnvDTE;

namespace BenjaminSchroeter.Flow.UiSupportPackage
{
    public static class ProjectExtensions
    {
        /// <summary>
        /// Determines whether the give project is a Scatter Web DSL project.
        /// </summary>
        /// <param name="project">The project.</param>
        /// <returns> 	<c>true</c> if the give project is a Scatter Web DSL project; otherwise, <c>false</c>. </returns>
        public static bool IsScatterWebDslProject(this Project project)
        {
            if (! project.IsCppProject() )
                return false;

            return project.Globals.get_VariableExists("ScatterWebDsl");
        }


        /// <summary>
        /// Determines whether the give project is a Scatter Web DSL project for Hardware Vendors.
        /// That is a Project with the ScatterWebDsl marker and a .hardware file present 
        /// </summary>
        /// <param name="project">The project.</param>
        /// <returns>
        /// 	<c>true</c> if the give project is a Scatter Web DSL project for Hardware Vendors; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsScatterWebDslHardwarePlatformProject(this Project project)
        {
            if (!project.IsScatterWebDslProject())
                return false;

            return project.FindAllFiles().Any(x => x.get_FileNames(0).EndsWith( Const.FileExtensionHardwareDescription, StringComparison.InvariantCultureIgnoreCase));
        }
    }
}
